@@ -1,12 +1,17 @@
-from __future__ import division, print_function
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
+import six
+
 import re
 import warnings
+import inspect
 import matplotlib
 import matplotlib.cbook as cbook
 from matplotlib import docstring, rcParams
-from transforms import Bbox, IdentityTransform, TransformedBbox, \
+from .transforms import Bbox, IdentityTransform, TransformedBbox, \
                        TransformedPath, Transform
-from path import Path
+from .path import Path
 
 ## Note, matplotlib artists use the doc strings for set and get
 # methods to enable the introspection methods of setp and getp.  Every
@@ -101,6 +106,8 @@ class Artist(object):
         self._url = None
         self._gid = None
         self._snap = None
+        self._sketch = rcParams['path.sketch']
+        self._path_effects = rcParams['path.effects']
 
     def __getstate__(self):
         d = self.__dict__.copy()
@@ -180,6 +187,23 @@ class Artist(object):
         """
         return self.axes
 
+    def get_window_extent(self, renderer):
+        """
+        Get the axes bounding box in display space.
+        Subclasses should override for inclusion in the bounding box
+        "tight" calculation. Default is to return an empty bounding
+        box at 0, 0.
+
+        Be careful when using this function, the results will not update
+        if the artist window extent of the artist changes.  The extent
+        can change due to any changes in the transform stack, such as
+        changing the axes limits, the figure size, or the canvas used
+        (as is done when saving a figure).  This can lead to unexpected
+        behavior where interactive figures will look fine on the screen,
+        but will save incorrectly.
+        """
+        return Bbox([[0, 0], [0, 0]])
+
     def add_callback(self, func):
         """
         Adds a callback function that will be called whenever one of
@@ -213,7 +237,7 @@ class Artist(object):
         Fire an event when property changed, calling all of the
         registered callbacks.
         """
-        for oid, func in self._propobservers.iteritems():
+        for oid, func in six.iteritems(self._propobservers):
             func(self)
 
     def is_transform_set(self):
@@ -278,7 +302,7 @@ class Artist(object):
         selection, such as which points are contained in the pick radius.  See
         individual artists for details.
         """
-        if callable(self._contains):
+        if six.callable(self._contains):
             return self._contains(self, mouseevent)
         warnings.warn("'%s' needs 'contains' method" % self.__class__.__name__)
         return False, {}
@@ -323,7 +347,7 @@ class Artist(object):
         # Pick self
         if self.pickable():
             picker = self.get_picker()
-            if callable(picker):
+            if six.callable(picker):
                 inside, prop = picker(self, mouseevent)
             else:
                 inside, prop = self.contains(mouseevent)
@@ -334,11 +358,14 @@ class Artist(object):
         for a in self.get_children():
             # make sure the event happened in the same axes
             ax = getattr(a, 'axes', None)
-            if mouseevent.inaxes is None or mouseevent.inaxes == ax:
+            if mouseevent.inaxes is None or ax is None or \
+                    mouseevent.inaxes == ax:
                 # we need to check if mouseevent.inaxes is None
-                # because some objects associated with an axes (eg a
+                # because some objects associated with an axes (e.g., a
                 # tick label) can be outside the bounding box of the
                 # axes and inaxes will be None
+                # also check that ax is None so that it traverse objects
+                # which do no have an axes property but children might
                 a.pick(mouseevent)
 
     def set_picker(self, picker):
@@ -358,7 +385,7 @@ class Artist(object):
             off an event if it's data is within epsilon of the mouse
             event.  For some artists like lines and patch collections,
             the artist may provide additional data to the pick event
-            that is generated, e.g. the indices of the data within
+            that is generated, e.g., the indices of the data within
             epsilon of the pick event
 
           * A function: if picker is callable, it is a user supplied
@@ -447,6 +474,63 @@ class Artist(object):
         """
         self._snap = snap
 
+    def get_sketch_params(self):
+        """
+        Returns the sketch parameters for the artist.
+
+        Returns
+        -------
+        sketch_params : tuple or `None`
+
+        A 3-tuple with the following elements:
+
+          * `scale`: The amplitude of the wiggle perpendicular to the
+            source line.
+
+          * `length`: The length of the wiggle along the line.
+
+          * `randomness`: The scale factor by which the length is
+            shrunken or expanded.
+
+        May return `None` if no sketch parameters were set.
+        """
+        return self._sketch
+
+    def set_sketch_params(self, scale=None, length=None, randomness=None):
+        """
+        Sets the the sketch parameters.
+
+        Parameters
+        ----------
+
+        scale : float, optional
+            The amplitude of the wiggle perpendicular to the source
+            line, in pixels.  If scale is `None`, or not provided, no
+            sketch filter will be provided.
+
+        length : float, optional
+             The length of the wiggle along the line, in pixels
+             (default 128.0)
+
+        randomness : float, optional
+            The scale factor by which the length is shrunken or
+            expanded (default 16.0)
+        """
+        if scale is None:
+            self._sketch = None
+        else:
+            self._sketch = (scale, length or 128.0, randomness or 16.0)
+
+    def set_path_effects(self, path_effects):
+        """
+        set path_effects, which should be a list of instances of
+        matplotlib.patheffect._Base class or its derivatives.
+        """
+        self._path_effects = path_effects
+
+    def get_path_effects(self):
+        return self._path_effects
+
     def get_figure(self):
         """
         Return the :class:`~matplotlib.figure.Figure` instance the
@@ -568,6 +652,9 @@ class Artist(object):
         """
         Set whether artist uses clipping.
 
+        When False artists will be visible out side of the axes which
+        can lead to unexpected results.
+
         ACCEPTS: [True | False]
         """
         self._clipon = b
@@ -663,10 +750,10 @@ class Artist(object):
         store = self.eventson
         self.eventson = False
         changed = False
-            
-        for k, v in props.iteritems():
+
+        for k, v in six.iteritems(props):
             func = getattr(self, 'set_' + k, None)
-            if func is None or not callable(func):
+            if func is None or not six.callable(func):
                 raise AttributeError('Unknown property %s' % k)
             func(v)
             changed = True
@@ -719,6 +806,8 @@ class Artist(object):
         self._clippath = other._clippath
         self._lod = other._lod
         self._label = other._label
+        self._sketch = other._sketch
+        self._path_effects = other._path_effects
         self.pchanged()
 
     def properties(self):
@@ -732,7 +821,7 @@ class Artist(object):
         A tkstyle set command, pass *kwargs* to set properties
         """
         ret = []
-        for k, v in kwargs.iteritems():
+        for k, v in six.iteritems(kwargs):
             k = k.lower()
             funcName = "set_%s" % k
             func = getattr(self, funcName)
@@ -743,10 +832,7 @@ class Artist(object):
         """
         Find artist objects.
 
-        pyplot signature:
-          findobj(o=gcf(), match=None, include_self=True)
-
-        Recursively find all :class:matplotlib.artist.Artist instances
+        Recursively find all :class:`~matplotlib.artist.Artist` instances
         contained in self.
 
         *match* can be
@@ -756,21 +842,19 @@ class Artist(object):
           - function with signature ``boolean = match(artist)``
             used to filter matches
 
-          - class instance: eg Line2D.  Only return artists of class type.
+          - class instance: e.g., Line2D.  Only return artists of class type.
 
         If *include_self* is True (default), include self in the list to be
         checked for a match.
 
-        .. plot:: mpl_examples/pylab_examples/findobj_demo.py
         """
-
         if match is None:  # always return True
             def matchfunc(x):
                 return True
         elif cbook.issubclass_safe(match, Artist):
             def matchfunc(x):
                 return isinstance(x, match)
-        elif callable(match):
+        elif six.callable(match):
             matchfunc = match
         else:
             raise ValueError('match must be None, a matplotlib.artist.Artist '
@@ -819,7 +903,7 @@ class ArtistInspector:
         Get a dict mapping *fullname* -> *alias* for each *alias* in
         the :class:`~matplotlib.artist.ArtistInspector`.
 
-        Eg., for lines::
+        e.g., for lines::
 
           {'markerfacecolor': 'mfc',
            'linewidth'      : 'lw',
@@ -828,7 +912,7 @@ class ArtistInspector:
         """
         names = [name for name in dir(self.o) if
                  (name.startswith('set_') or name.startswith('get_'))
-                 and callable(getattr(self.o, name))]
+                 and six.callable(getattr(self.o, name))]
         aliases = {}
         for name in names:
             func = getattr(self.o, name)
@@ -849,7 +933,7 @@ class ArtistInspector:
         This is done by querying the docstring of the function *set_attr*
         for a line that begins with ACCEPTS:
 
-        Eg., for a line linestyle, return
+        e.g., for a line linestyle, return
         "[ ``'-'`` | ``'--'`` | ``'-.'`` | ``':'`` | ``'steps'`` | ``'None'`` ]"
         """
 
@@ -881,7 +965,9 @@ class ArtistInspector:
             if not name.startswith('set_'):
                 continue
             o = getattr(self.o, name)
-            if not callable(o):
+            if not six.callable(o):
+                continue
+            if len(inspect.getargspec(o)[0]) < 2:
                 continue
             func = o
             if self.is_alias(func):
@@ -896,7 +982,7 @@ class ArtistInspector:
 
     def get_setters(self):
         """
-        Get the attribute strings with setters for object.  Eg., for a line,
+        Get the attribute strings with setters for object.  e.g., for a line,
         return ``['markerfacecolor', 'linewidth', ....]``.
         """
 
@@ -917,7 +1003,7 @@ class ArtistInspector:
         return 'PROPNAME or alias' if *s* has an alias, else return
         PROPNAME.
 
-        E.g. for the line markerfacecolor property, which has an
+        e.g., for the line markerfacecolor property, which has an
         alias, return 'markerfacecolor or mfc' and for the transform
         property, which does not, return 'transform'
         """
@@ -925,7 +1011,7 @@ class ArtistInspector:
         if s in self.aliasd:
             return s + ''.join([' or %s' % x
                                 for x
-                                in self.aliasd[s].iterkeys()])
+                                in six.iterkeys(self.aliasd[s])])
         else:
             return s
 
@@ -934,7 +1020,7 @@ class ArtistInspector:
         return 'PROPNAME or alias' if *s* has an alias, else return
         PROPNAME formatted for ReST
 
-        E.g. for the line markerfacecolor property, which has an
+        e.g., for the line markerfacecolor property, which has an
         alias, return 'markerfacecolor or mfc' and for the transform
         property, which does not, return 'transform'
         """
@@ -942,7 +1028,7 @@ class ArtistInspector:
         if s in self.aliasd:
             aliases = ''.join([' or %s' % x
                                for x
-                               in self.aliasd[s].iterkeys()])
+                               in six.iterkeys(self.aliasd[s])])
         else:
             aliases = ''
         return ':meth:`%s <%s>`%s' % (s, target, aliases)
@@ -1034,7 +1120,7 @@ class ArtistInspector:
         o = self.oorig
         getters = [name for name in dir(o)
                    if name.startswith('get_')
-                   and callable(getattr(o, name))]
+                   and six.callable(getattr(o, name))]
         #print getters
         getters.sort()
         d = dict()
@@ -1058,7 +1144,7 @@ class ArtistInspector:
         """
 
         d = self.properties()
-        names = d.keys()
+        names = list(six.iterkeys(d))
         names.sort()
         lines = []
         for name in names:
@@ -1083,7 +1169,7 @@ class ArtistInspector:
 
           - function with signature ``boolean = match(artist)``
 
-          - class instance: eg :class:`~matplotlib.lines.Line2D`
+          - class instance: e.g., :class:`~matplotlib.lines.Line2D`
 
         used to filter matches.
         """
@@ -1093,7 +1179,7 @@ class ArtistInspector:
         elif issubclass(match, Artist):
             def matchfunc(x):
                 return isinstance(x, match)
-        elif callable(match):
+        elif six.callable(match):
             matchfunc = func
         else:
             raise ValueError('match must be None, an '
@@ -1125,7 +1211,7 @@ def getp(obj, property=None):
         getp(obj)  # get all the object properties
         getp(obj, 'linestyle')  # get the linestyle property
 
-    *obj* is a :class:`Artist` instance, eg
+    *obj* is a :class:`Artist` instance, e.g.,
     :class:`~matplotllib.lines.Line2D` or an instance of a
     :class:`~matplotlib.axes.Axes` or :class:`matplotlib.text.Text`.
     If the *property* is 'somename', this function returns
@@ -1183,7 +1269,7 @@ def setp(obj, *args, **kwargs):
     :func:`setp` operates on a single instance or a list of instances.
     If you are in query mode introspecting the possible values, only
     the first instance in the sequence is used.  When actually setting
-    values, all the instances will be set.  E.g., suppose you have a
+    values, all the instances will be set.  e.g., suppose you have a
     list of two lines, the following will make both lines thicker and
     red::
 
@@ -1196,9 +1282,8 @@ def setp(obj, *args, **kwargs):
     :func:`setp` works with the MATLAB style string/value pairs or
     with python kwargs.  For example, the following are equivalent::
 
-      >>> setp(lines, 'linewidth', 2, 'color', r')  # MATLAB style
-
-      >>> setp(lines, linewidth=2, color='r')       # python style
+      >>> setp(lines, 'linewidth', 2, 'color', 'r')  # MATLAB style
+      >>> setp(lines, linewidth=2, color='r')        # python style
     """
 
     insp = ArtistInspector(obj)
@@ -1222,7 +1307,7 @@ def setp(obj, *args, **kwargs):
     funcvals = []
     for i in range(0, len(args) - 1, 2):
         funcvals.append((args[i], args[i + 1]))
-    funcvals.extend(kwargs.iteritems())
+    funcvals.extend(kwargs.items())
 
     ret = []
     for o in objs:
