@@ -8,7 +8,7 @@ Features that are implemented
  * linewidth
  * lines, rectangles, ellipses
  * clipping to a rectangle
- * output to RGBA and PNG
+ * output to RGBA and PNG, optionally JPEG and TIFF
  * alpha blending
  * DPI scaling properly - everything scales properly (dashes, linewidths, etc)
  * draw polygon
@@ -16,18 +16,21 @@ Features that are implemented
 
 TODO:
 
-  * allow save to file handle
-
   * integrate screen dpi w/ ppi and text
+
 """
-from __future__ import division
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
+import six
+
 import threading
 import numpy as np
 
 from matplotlib import verbose, rcParams
 from matplotlib.backend_bases import RendererBase,\
      FigureManagerBase, FigureCanvasBase
-from matplotlib.cbook import is_string_like, maxdict
+from matplotlib.cbook import is_string_like, maxdict, restrict_dict
 from matplotlib.figure import Figure
 from matplotlib.font_manager import findfont
 from matplotlib.ft2font import FT2Font, LOAD_FORCE_AUTOHINT, LOAD_NO_HINTING, \
@@ -39,6 +42,12 @@ from matplotlib.transforms import Bbox, BboxBase
 from matplotlib.backends._backend_agg import RendererAgg as _RendererAgg
 from matplotlib import _png
 
+try:
+    from PIL import Image
+    _has_pil = True
+except ImportError:
+    _has_pil = False
+
 backend_version = 'v2.2'
 
 def get_hinting_flag():
@@ -95,6 +104,14 @@ class RendererAgg(RendererBase):
         if __debug__: verbose.report('RendererAgg.__init__ done',
                                      'debug-annoying')
 
+    def __getstate__(self):
+        # We only want to preserve the init keywords of the Renderer.
+        # Anything else can be re-created.
+        return {'width': self.width, 'height': self.height, 'dpi': self.dpi}
+
+    def __setstate__(self, state):
+        self.__init__(state['width'], state['height'], state['dpi'])
+
     def _get_hinting_flag(self):
         if rcParams['text.hinting']:
             return LOAD_FORCE_AUTOHINT
@@ -127,7 +144,7 @@ class RendererAgg(RendererBase):
         nmax = rcParams['agg.path.chunksize'] # here at least for testing
         npts = path.vertices.shape[0]
         if (nmax > 100 and npts > nmax and path.should_simplify and
-            rgbFace is None and gc.get_hatch() is None):
+                rgbFace is None and gc.get_hatch() is None):
             nch = np.ceil(npts/float(nmax))
             chsize = int(np.ceil(npts/nch))
             i0 = np.arange(0, npts, chsize)
@@ -154,11 +171,13 @@ class RendererAgg(RendererBase):
         ox, oy, width, height, descent, font_image, used_characters = \
             self.mathtext_parser.parse(s, self.dpi, prop)
 
-        x = np.round(x + ox)
-        y = np.round(y - oy)
+        xd = descent * np.sin(np.deg2rad(angle))
+        yd = descent * np.cos(np.deg2rad(angle))
+        x = np.round(x + ox + xd)
+        y = np.round(y - oy + yd)
         self._renderer.draw_text_image(font_image, x, y + 1, angle, gc)
 
-    def draw_text(self, gc, x, y, s, prop, angle, ismath):
+    def draw_text(self, gc, x, y, s, prop, angle, ismath=False, mtext=None):
         """
         Render the text
         """
@@ -177,17 +196,21 @@ class RendererAgg(RendererBase):
             # space) in the following call to draw_text_image).
             font.set_text(s, 0, flags=flags)
         font.draw_glyphs_to_bitmap(antialiased=rcParams['text.antialiased'])
+        d = font.get_descent() / 64.0
+        # The descent needs to be adjusted for the angle
+        xd = -d * np.sin(np.deg2rad(angle))
+        yd = d * np.cos(np.deg2rad(angle))
 
         #print x, y, int(x), int(y), s
         self._renderer.draw_text_image(
-            font.get_image(), np.round(x), np.round(y) + 1, angle, gc)
+            font.get_image(), np.round(x - xd), np.round(y + yd) + 1, angle, gc)
 
     def get_text_width_height_descent(self, s, prop, ismath):
         """
         get the width and height in display coords of the string s
         with FontPropertry prop
 
-        # passing rgb is a little hack to make cacheing in the
+        # passing rgb is a little hack to make caching in the
         # texmanager more efficient.  It is not meant to be used
         # outside the backend
         """
@@ -216,7 +239,7 @@ class RendererAgg(RendererBase):
         return w, h, d
 
 
-    def draw_tex(self, gc, x, y, s, prop, angle):
+    def draw_tex(self, gc, x, y, s, prop, angle, ismath='TeX!', mtext=None):
         # todo, handle props, angle, origins
         size = prop.get_size_in_points()
 
@@ -227,6 +250,12 @@ class RendererAgg(RendererBase):
             Z = texmanager.get_grey(s, size, self.dpi)
             Z = np.array(Z * 255.0, np.uint8)
 
+        w, h, d = self.get_text_width_height_descent(s, prop, ismath)
+        xd = d * np.sin(np.deg2rad(angle))
+        yd = d * np.cos(np.deg2rad(angle))
+        x = np.round(x + xd)
+        y = np.round(y + yd)
+
         self._renderer.draw_text_image(Z, x, y, angle, gc)
 
     def get_canvas_width_height(self):
@@ -248,7 +277,7 @@ class RendererAgg(RendererBase):
             font = RendererAgg._fontd.get(fname)
             if font is None:
                 font = FT2Font(
-                    str(fname),
+                    fname,
                     hinting_factor=rcParams['text.hinting_factor'])
                 RendererAgg._fontd[fname] = font
             RendererAgg._fontd[key] = font
@@ -309,7 +338,7 @@ class RendererAgg(RendererBase):
         >>> region = renderer.copy_from_bbox()
         >>> x1, y1, x2, y2 = region.get_extents()
         >>> renderer.restore_region(region, bbox=(x1+dx, y1, x2, y2),
-                                    xy=(x1-dx, y1))
+        ...                         xy=(x1-dx, y1))
 
         """
         if bbox is not None or xy is not None:
@@ -432,7 +461,7 @@ class FigureCanvasAgg(FigureCanvasBase):
         """
         if __debug__: verbose.report('FigureCanvasAgg.draw', 'debug-annoying')
 
-        self.renderer = self.get_renderer()
+        self.renderer = self.get_renderer(cleared=True)
         # acquire a lock on the shared font cache
         RendererAgg.lock.acquire()
 
@@ -441,9 +470,7 @@ class FigureCanvasAgg(FigureCanvasBase):
         finally:
             RendererAgg.lock.release()
 
-
-
-    def get_renderer(self):
+    def get_renderer(self, cleared=False):
         l, b, w, h = self.figure.bbox.bounds
         key = w, h, self.figure.dpi
         try: self._lastKey, self.renderer
@@ -453,6 +480,8 @@ class FigureCanvasAgg(FigureCanvasBase):
         if need_new_renderer:
             self.renderer = RendererAgg(w, h, self.figure.dpi)
             self._lastKey = key
+        elif cleared:
+            self.renderer.clear()
         return self.renderer
 
     def tostring_rgb(self):
@@ -516,3 +545,50 @@ class FigureCanvasAgg(FigureCanvasBase):
                   (int(renderer.width), int(renderer.height)))
         renderer.dpi = original_dpi
         return result
+
+    if _has_pil:
+
+        # add JPEG support
+        def print_jpg(self, filename_or_obj, *args, **kwargs):
+            """
+            Supported kwargs:
+
+            *quality*: The image quality, on a scale from 1 (worst) to
+                95 (best). The default is 95, if not given in the
+                matplotlibrc file in the savefig.jpeg_quality parameter.
+                Values above 95 should be avoided; 100 completely
+                disables the JPEG quantization stage.
+
+            *optimize*: If present, indicates that the encoder should
+                make an extra pass over the image in order to select
+                optimal encoder settings.
+
+            *progressive*: If present, indicates that this image
+                should be stored as a progressive JPEG file.
+            """
+            buf, size = self.print_to_buffer()
+            if kwargs.pop("dryrun", False):
+                return
+            image = Image.frombuffer('RGBA', size, buf, 'raw', 'RGBA', 0, 1)
+            options = restrict_dict(kwargs, ['quality', 'optimize',
+                                             'progressive'])
+
+            if 'quality' not in options:
+                options['quality'] = rcParams['savefig.jpeg_quality']
+
+            return image.save(filename_or_obj, format='jpeg', **options)
+        print_jpeg = print_jpg
+
+        # add TIFF support
+        def print_tif(self, filename_or_obj, *args, **kwargs):
+            buf, size = self.print_to_buffer()
+            if kwargs.pop("dryrun", False):
+                return
+            image = Image.frombuffer('RGBA', size, buf, 'raw', 'RGBA', 0, 1)
+            dpi = (self.figure.dpi, self.figure.dpi)
+            return image.save(filename_or_obj, format='tiff',
+                              dpi=dpi)
+        print_tiff = print_tif
+
+
+FigureCanvas = FigureCanvasAgg
