@@ -1,8 +1,17 @@
 """
-A collection of utility functions and classes.  Many (but not all)
-from the Python Cookbook -- hence the name cbook
+A collection of utility functions and classes.  Originally, many
+(but not all) were from the Python Cookbook -- hence the name cbook.
+
+This module is safe to import from anywhere within matplotlib;
+it imports matplotlib only at runtime.
 """
-from __future__ import print_function
+
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
+import six
+from six.moves import xrange, zip
+from itertools import repeat
 
 import datetime
 import errno
@@ -13,25 +22,227 @@ import io
 import locale
 import os
 import re
-import subprocess
 import sys
 import threading
 import time
 import traceback
+import types
 import warnings
 from weakref import ref, WeakKeyDictionary
 
-import matplotlib
-from matplotlib import MatplotlibDeprecationWarning as mplDeprecation
-
 import numpy as np
 import numpy.ma as ma
 
 
-if sys.version_info[0] >= 3:
-    import types
-else:
-    import new
+class MatplotlibDeprecationWarning(UserWarning):
+    """
+    A class for issuing deprecation warnings for Matplotlib users.
+
+    In light of the fact that Python builtin DeprecationWarnings are ignored
+    by default as of Python 2.7 (see link below), this class was put in to
+    allow for the signaling of deprecation, but via UserWarnings which are not
+    ignored by default.
+
+    http://docs.python.org/dev/whatsnew/2.7.html#the-future-for-python-2-x
+    """
+    pass
+
+mplDeprecation = MatplotlibDeprecationWarning
+
+
+def _generate_deprecation_message(since, message='', name='',
+                                  alternative='', pending=False,
+                                  obj_type='attribute'):
+
+    if not message:
+        altmessage = ''
+
+        if pending:
+            message = (
+                'The %(func)s %(obj_type)s will be deprecated in a '
+                'future version.')
+        else:
+            message = (
+                'The %(func)s %(obj_type)s was deprecated in version '
+                '%(since)s.')
+        if alternative:
+            altmessage = ' Use %s instead.' % alternative
+
+        message = ((message % {
+            'func': name,
+            'name': name,
+            'alternative': alternative,
+            'obj_type': obj_type,
+            'since': since}) +
+            altmessage)
+
+    return message
+
+
+def warn_deprecated(
+        since, message='', name='', alternative='', pending=False,
+        obj_type='attribute'):
+    """
+    Used to display deprecation warning in a standard way.
+
+    Parameters
+    ------------
+    since : str
+        The release at which this API became deprecated.
+
+    message : str, optional
+        Override the default deprecation message.  The format
+        specifier `%(func)s` may be used for the name of the function,
+        and `%(alternative)s` may be used in the deprecation message
+        to insert the name of an alternative to the deprecated
+        function.  `%(obj_type)` may be used to insert a friendly name
+        for the type of object being deprecated.
+
+    name : str, optional
+        The name of the deprecated function; if not provided the name
+        is automatically determined from the passed in function,
+        though this is useful in the case of renamed functions, where
+        the new function is just assigned to the name of the
+        deprecated function.  For example::
+
+            def new_function():
+                ...
+            oldFunction = new_function
+
+    alternative : str, optional
+        An alternative function that the user may use in place of the
+        deprecated function.  The deprecation warning will tell the user about
+        this alternative if provided.
+
+    pending : bool, optional
+        If True, uses a PendingDeprecationWarning instead of a
+        DeprecationWarning.
+
+    obj_type : str, optional
+        The object type being deprecated.
+
+    Examples
+    --------
+
+        Basic example::
+
+            # To warn of the deprecation of "matplotlib.name_of_module"
+            warn_deprecated('1.4.0', name='matplotlib.name_of_module',
+                            obj_type='module')
+
+    """
+    message = _generate_deprecation_message(
+        since, message, name, alternative, pending, obj_type)
+
+    warnings.warn(message, mplDeprecation, stacklevel=1)
+
+
+def deprecated(since, message='', name='', alternative='', pending=False,
+               obj_type='function'):
+    """
+    Decorator to mark a function as deprecated.
+
+    Parameters
+    ------------
+    since : str
+        The release at which this API became deprecated.  This is
+        required.
+
+    message : str, optional
+        Override the default deprecation message.  The format
+        specifier `%(func)s` may be used for the name of the function,
+        and `%(alternative)s` may be used in the deprecation message
+        to insert the name of an alternative to the deprecated
+        function.  `%(obj_type)` may be used to insert a friendly name
+        for the type of object being deprecated.
+
+    name : str, optional
+        The name of the deprecated function; if not provided the name
+        is automatically determined from the passed in function,
+        though this is useful in the case of renamed functions, where
+        the new function is just assigned to the name of the
+        deprecated function.  For example::
+
+            def new_function():
+                ...
+            oldFunction = new_function
+
+    alternative : str, optional
+        An alternative function that the user may use in place of the
+        deprecated function.  The deprecation warning will tell the user about
+        this alternative if provided.
+
+    pending : bool, optional
+        If True, uses a PendingDeprecationWarning instead of a
+        DeprecationWarning.
+
+    Examples
+    --------
+
+        Basic example::
+
+            @deprecated('1.4.0')
+            def the_function_to_deprecate():
+                pass
+
+    """
+    def deprecate(func, message=message, name=name, alternative=alternative,
+                  pending=pending):
+        import functools
+        import textwrap
+
+        if isinstance(func, classmethod):
+            try:
+                func = func.__func__
+            except AttributeError:
+                # classmethods in Python2.6 and below lack the __func__
+                # attribute so we need to hack around to get it
+                method = func.__get__(None, object)
+                if hasattr(method, '__func__'):
+                    func = method.__func__
+                elif hasattr(method, 'im_func'):
+                    func = method.im_func
+                else:
+                    # Nothing we can do really...  just return the original
+                    # classmethod
+                    return func
+            is_classmethod = True
+        else:
+            is_classmethod = False
+
+        if not name:
+            name = func.__name__
+
+        message = _generate_deprecation_message(
+            since, message, name, alternative, pending, obj_type)
+
+        @functools.wraps(func)
+        def deprecated_func(*args, **kwargs):
+            warnings.warn(message, mplDeprecation, stacklevel=2)
+
+            return func(*args, **kwargs)
+
+        old_doc = deprecated_func.__doc__
+        if not old_doc:
+            old_doc = ''
+        old_doc = textwrap.dedent(old_doc).strip('\n')
+        message = message.strip()
+        new_doc = (('\n.. deprecated:: %(since)s'
+                    '\n    %(message)s\n\n' %
+                    {'since': since, 'message': message}) + old_doc)
+        if not old_doc:
+            # This is to prevent a spurious 'unexected unindent' warning from
+            # docutils when the original docstring was blank.
+            new_doc += r'\ '
+
+        deprecated_func.__doc__ = new_doc
+
+        if is_classmethod:
+            deprecated_func = classmethod(deprecated_func)
+        return deprecated_func
+
+    return deprecate
+
 
 # On some systems, locale.getpreferredencoding returns None,
 # which can break unicode; and the sage project reports that
@@ -42,30 +253,10 @@ else:
 # On some systems, getpreferredencoding sets the locale, which has
 # side effects.  Passing False eliminates those side effects.
 
-if sys.version_info[0] >= 3:
-    def unicode_safe(s):
-        import matplotlib
-
-        try:
-            preferredencoding = locale.getpreferredencoding(
-                matplotlib.rcParams['axes.formatter.use_locale']).strip()
-            if not preferredencoding:
-                preferredencoding = None
-        except (ValueError, ImportError, AttributeError):
-            preferredencoding = None
-
-        if isinstance(s, bytes):
-            if preferredencoding is None:
-                return unicode(s)
-            else:
-                # We say "unicode" and not "str" here so it passes through
-                # 2to3 correctly.
-                return unicode(s, preferredencoding)
-        return s
-else:
-    def unicode_safe(s):
-        import matplotlib
+def unicode_safe(s):
+    import matplotlib
 
+    if isinstance(s, bytes):
         try:
             preferredencoding = locale.getpreferredencoding(
                 matplotlib.rcParams['axes.formatter.use_locale']).strip()
@@ -75,12 +266,13 @@ else:
             preferredencoding = None
 
         if preferredencoding is None:
-            return unicode(s)
+            return six.text_type(s)
         else:
-            return unicode(s, preferredencoding)
+            return six.text_type(s, preferredencoding)
+    return s
 
 
-class converter:
+class converter(object):
     """
     Base class for handling string -> python type with support for
     missing values
@@ -174,8 +366,12 @@ class _BoundMethodProxy(object):
                 self.inst = ref(cb.im_self)
             except TypeError:
                 self.inst = None
-            self.func = cb.im_func
-            self.klass = cb.im_class
+            if six.PY3:
+                self.func = cb.__func__
+                self.klass = cb.__self__.__class__
+            else:
+                self.func = cb.im_func
+                self.klass = cb.im_class
         except AttributeError:
             self.inst = None
             self.func = cb
@@ -209,10 +405,9 @@ class _BoundMethodProxy(object):
         elif self.inst is not None:
             # build a new instance method with a strong reference to the
             # instance
-            if sys.version_info[0] >= 3:
-                mtd = types.MethodType(self.func, self.inst())
-            else:
-                mtd = new.instancemethod(self.func, self.inst(), self.klass)
+
+            mtd = types.MethodType(self.func, self.inst())
+
         else:
             # not a bound method, just return the func
             mtd = self.func
@@ -245,9 +440,9 @@ class CallbackRegistry:
     callbacks:
 
         >>> def oneat(x):
-        ...    print 'eat', x
+        ...    print('eat', x)
         >>> def ondrink(x):
-        ...    print 'drink', x
+        ...    print('drink', x)
 
         >>> from matplotlib.cbook import CallbackRegistry
         >>> callbacks = CallbackRegistry()
@@ -279,10 +474,11 @@ class CallbackRegistry:
     """
     def __init__(self, *args):
         if len(args):
-            warnings.warn(
-                'CallbackRegistry no longer requires a list of callback types.'
-                ' Ignoring arguments',
-                mplDeprecation)
+            warn_deprecated(
+                '1.3',
+                message="CallbackRegistry no longer requires a list of "
+                        "callback types. Ignoring arguments. *args will "
+                        "be removed in 1.5")
         self.callbacks = dict()
         self._cid = 0
         self._func_cid_map = {}
@@ -317,14 +513,15 @@ class CallbackRegistry:
         """
         disconnect the callback registered with callback id *cid*
         """
-        for eventname, callbackd in self.callbacks.items():
+        for eventname, callbackd in list(six.iteritems(self.callbacks)):
             try:
                 del callbackd[cid]
             except KeyError:
                 continue
             else:
-                for category, functions in self._func_cid_map.items():
-                    for function, value in functions.items():
+                for category, functions in list(
+                        six.iteritems(self._func_cid_map)):
+                    for function, value in list(six.iteritems(functions)):
                         if value == cid:
                             del functions[function]
                 return
@@ -335,7 +532,7 @@ class CallbackRegistry:
         callbacks on *s* will be called with *\*args* and *\*\*kwargs*
         """
         if s in self.callbacks:
-            for cid, proxy in self.callbacks[s].items():
+            for cid, proxy in list(six.iteritems(self.callbacks[s])):
                 # Clean out dead references
                 if proxy.inst is not None and proxy.inst() is None:
                     del self.callbacks[s][cid]
@@ -461,7 +658,7 @@ class Bunch:
         self.__dict__.update(kwds)
 
     def __repr__(self):
-        keys = self.__dict__.iterkeys()
+        keys = six.iterkeys(self.__dict__)
         return 'Bunch(%s)' % ', '.join(['%s=%s' % (k, self.__dict__[k])
                                         for k
                                         in keys])
@@ -469,7 +666,7 @@ class Bunch:
 
 def unique(x):
     'Return a list of unique elements of *x*'
-    return dict([(val, 1) for val in x]).keys()
+    return list(six.iterkeys(dict([(val, 1) for val in x])))
 
 
 def iterable(obj):
@@ -483,7 +680,7 @@ def iterable(obj):
 
 def is_string_like(obj):
     'Return True if *obj* looks like a string'
-    if isinstance(obj, (str, unicode)):
+    if isinstance(obj, six.string_types):
         return True
     # numpy strings are subclass of str, ma strings are not
     if ma.isMaskedArray(obj):
@@ -514,7 +711,20 @@ def is_sequence_of_strings(obj):
 
 def is_writable_file_like(obj):
     'return true if *obj* looks like a file object with a *write* method'
-    return hasattr(obj, 'write') and callable(obj.write)
+    return hasattr(obj, 'write') and six.callable(obj.write)
+
+
+def file_requires_unicode(x):
+    """
+    Returns `True` if the given writable file-like object requires Unicode
+    to be written to it.
+    """
+    try:
+        x.write(b'')
+    except TypeError:
+        return True
+    else:
+        return False
 
 
 def is_scalar(obj):
@@ -540,7 +750,6 @@ def to_filehandle(fname, flag='rU', return_opened=False):
     """
     if is_string_like(fname):
         if fname.endswith('.gz'):
-            import gzip
             # get rid of 'U' in flag for gzipped files.
             flag = flag.replace('U', '')
             fh = gzip.open(fname, flag)
@@ -567,6 +776,16 @@ def is_scalar_or_string(val):
     return is_string_like(val) or not iterable(val)
 
 
+def _string_to_bool(s):
+    if not is_string_like(s):
+        return s
+    if s == 'on':
+        return True
+    if s == 'off':
+        return False
+    raise ValueError("string argument must be either 'on' or 'off'")
+
+
 def get_sample_data(fname, asfileobj=True):
     """
     Return a sample data file.  *fname* is a path relative to the
@@ -580,10 +799,13 @@ def get_sample_data(fname, asfileobj=True):
 
     If the filename ends in .gz, the file is implicitly ungzipped.
     """
+    import matplotlib
+
     if matplotlib.rcParams['examples.directory']:
         root = matplotlib.rcParams['examples.directory']
     else:
-        root = os.path.join(os.path.dirname(__file__), "mpl-data", "sample_data")
+        root = os.path.join(os.path.dirname(__file__),
+                            "mpl-data", "sample_data")
     path = os.path.join(root, fname)
 
     if asfileobj:
@@ -610,7 +832,7 @@ def flatten(seq, scalarp=is_scalar_or_string):
 
         >>> from matplotlib.cbook import flatten
         >>> l = (('John', ['Hunter']), (1, 23), [[([42, (5, 23)], )]])
-        >>> print list(flatten(l))
+        >>> print(list(flatten(l)))
         ['John', 'Hunter', 1, 23, 42, 5, 23]
 
     By: Composite of Holger Krekel and Luther Blissett
@@ -694,7 +916,7 @@ class Xlator(dict):
 
     def _make_regex(self):
         """ Build re object based on the keys of the current dictionary """
-        return re.compile("|".join(map(re.escape, self.iterkeys())))
+        return re.compile("|".join(map(re.escape, list(six.iterkeys(self)))))
 
     def __call__(self, match):
         """ Handler invoked for each regex *match* """
@@ -748,8 +970,12 @@ class Null:
     def __repr__(self):
         return "Null()"
 
-    def __nonzero__(self):
-        return 0
+    if six.PY3:
+        def __bool__(self):
+            return 0
+    else:
+        def __nonzero__(self):
+            return 0
 
     def __getattr__(self, name):
         return self
@@ -987,8 +1213,10 @@ def pieces(seq, num=2):
 
 
 def exception_to_str(s=None):
-
-    sh = io.StringIO()
+    if six.PY3:
+        sh = io.StringIO()
+    else:
+        sh = io.BytesIO()
     if s is not None:
         print(s, file=sh)
     traceback.print_exc(file=sh)
@@ -1184,7 +1412,7 @@ def finddir(o, match, case=False):
 
 def reverse_dict(d):
     'reverse the dictionary -- may lose data if values are not unique!'
-    return dict([(v, k) for k, v in d.iteritems()])
+    return dict([(v, k) for k, v in six.iteritems(d)])
 
 
 def restrict_dict(d, keys):
@@ -1192,24 +1420,39 @@ def restrict_dict(d, keys):
     Return a dictionary that contains those keys that appear in both
     d and keys, with values from d.
     """
-    return dict([(k, v) for (k, v) in d.iteritems() if k in keys])
+    return dict([(k, v) for (k, v) in six.iteritems(d) if k in keys])
 
 
 def report_memory(i=0):  # argument may go away
     'return the memory consumed by process'
-    from subprocess import Popen, PIPE
+    from matplotlib.compat.subprocess import Popen, PIPE
     pid = os.getpid()
     if sys.platform == 'sunos5':
-        a2 = Popen('ps -p %d -o osz' % pid, shell=True,
-                   stdout=PIPE).stdout.readlines()
+        try:
+            a2 = Popen('ps -p %d -o osz' % pid, shell=True,
+                       stdout=PIPE).stdout.readlines()
+        except OSError:
+            raise NotImplementedError(
+                "report_memory works on Sun OS only if "
+                "the 'ps' program is found")
         mem = int(a2[-1].strip())
     elif sys.platform.startswith('linux'):
-        a2 = Popen('ps -p %d -o rss,sz' % pid, shell=True,
-                   stdout=PIPE).stdout.readlines()
+        try:
+            a2 = Popen('ps -p %d -o rss,sz' % pid, shell=True,
+                       stdout=PIPE).stdout.readlines()
+        except OSError:
+            raise NotImplementedError(
+                "report_memory works on Linux only if "
+                "the 'ps' program is found")
         mem = int(a2[1].split()[1])
     elif sys.platform.startswith('darwin'):
-        a2 = Popen('ps -p %d -o rss,vsz' % pid, shell=True,
-                   stdout=PIPE).stdout.readlines()
+        try:
+            a2 = Popen('ps -p %d -o rss,vsz' % pid, shell=True,
+                       stdout=PIPE).stdout.readlines()
+        except OSError:
+            raise NotImplementedError(
+                "report_memory works on Mac OS only if "
+                "the 'ps' program is found")
         mem = int(a2[1].split()[0])
     elif sys.platform.startswith('win'):
         try:
@@ -1234,7 +1477,7 @@ def safezip(*args):
     for i, arg in enumerate(args[1:]):
         if len(arg) != Nx:
             raise ValueError(_safezip_msg % (Nx, i + 1, len(arg)))
-    return zip(*args)
+    return list(zip(*args))
 
 
 def issubclass_safe(x, klass):
@@ -1279,7 +1522,7 @@ class MemoryMonitor:
         n = self._n
         segments = min(n, segments)
         dn = int(n / segments)
-        ii = range(0, n, dn)
+        ii = list(xrange(0, n, dn))
         ii[-1] = n - 1
         print()
         print('memory report: i, mem, dmem, dmem/nloops')
@@ -1300,7 +1543,7 @@ class MemoryMonitor:
 
     def plot(self, i0=0, isub=1, fig=None):
         if fig is None:
-            from pylab import figure, show
+            from .pylab import figure
             fig = figure()
 
         ax = fig.add_subplot(111)
@@ -1331,7 +1574,7 @@ def print_cycles(objects, outstream=sys.stdout, show_progress=False):
 
             outstream.write("   %s -- " % str(type(step)))
             if isinstance(step, dict):
-                for key, val in step.iteritems():
+                for key, val in six.iteritems(step):
                     if val is next:
                         outstream.write("[%s]" % repr(key))
                         break
@@ -1402,7 +1645,7 @@ class Grouper(object):
         >>> grp.join(b, c)
         >>> grp.join(d, e)
         >>> sorted(map(tuple, grp))
-        [(d, e), (a, b, c)]
+        [(a, b, c), (d, e)]
         >>> grp.joined(a, b)
         True
         >>> grp.joined(a, c)
@@ -1477,13 +1720,13 @@ class Grouper(object):
 
         # Mark each group as we come across if by appending a token,
         # and don't yield it twice
-        for group in self._mapping.itervalues():
+        for group in six.itervalues(self._mapping):
             if not group[-1] is token:
                 yield [x() for x in group]
                 group.append(token)
 
         # Cleanup the tokens
-        for group in self._mapping.itervalues():
+        for group in six.itervalues(self._mapping):
             if group[-1] is token:
                 del group[-1]
 
@@ -1501,7 +1744,7 @@ def simple_linear_interpolation(a, steps):
     if steps == 1:
         return a
 
-    steps = np.floor(steps)
+    steps = int(np.floor(steps))
     new_length = ((len(a) - 1) * steps) + 1
     new_shape = list(a.shape)
     new_shape[0] = new_length
@@ -1511,8 +1754,7 @@ def simple_linear_interpolation(a, steps):
     a0 = a[0:-1]
     a1 = a[1:]
     delta = ((a1 - a0) / steps)
-
-    for i in range(1, int(steps)):
+    for i in range(1, steps):
         result[i::steps] = delta * i + a0
     result[steps::steps] = a1
 
@@ -1521,8 +1763,8 @@ def simple_linear_interpolation(a, steps):
 
 def recursive_remove(path):
     if os.path.isdir(path):
-        for fname in glob.glob(os.path.join(path, '*')) + \
-                     glob.glob(os.path.join(path, '.*')):
+        for fname in (glob.glob(os.path.join(path, '*')) +
+                      glob.glob(os.path.join(path, '.*'))):
             if os.path.isdir(fname):
                 recursive_remove(fname)
                 os.removedirs(fname)
@@ -1610,8 +1852,208 @@ def delete_masked_points(*args):
     return margs
 
 
-def unmasked_index_ranges(mask, compressed=True):
+def boxplot_stats(X, whis=1.5, bootstrap=None, labels=None):
     '''
+    Returns list of dictionaries of staticists to be use to draw a series of
+    box and whisker plots. See the `Returns` section below to the required
+    keys of the dictionary. Users can skip this function and pass a user-
+    defined set of dictionaries to the new `axes.bxp` method instead of
+    relying on MPL to do the calcs.
+
+    Parameters
+    ----------
+    X : array-like
+        Data that will be represented in the boxplots. Should have 2 or fewer
+        dimensions.
+
+    whis : float, string, or sequence (default = 1.5)
+        As a float, determines the reach of the whiskers past the first and
+        third quartiles (e.g., Q3 + whis*IQR, QR = interquartile range, Q3-Q1).
+        Beyond the whiskers, data are considered outliers and are plotted as
+        individual points. Set this to an unreasonably high value to force the
+        whiskers to show the min and max data. Alternatively, set this to an
+        ascending sequence of percentile (e.g., [5, 95]) to set the whiskers
+        at specific percentiles of the data. Finally, can  `whis` be the
+        string 'range' to force the whiskers to the min and max of the data.
+        In the edge case that the 25th and 75th percentiles are equivalent,
+        `whis` will be automatically set to 'range'
+
+    bootstrap : int or None (default)
+        Number of times the confidence intervals around the median should
+        be bootstrapped (percentile method).
+
+    labels : sequence
+        Labels for each dataset. Length must be compatible with dimensions
+        of `X`
+
+    Returns
+    -------
+    bxpstats : list of dict
+        A list of dictionaries containing the results for each column
+        of data. Keys of each dictionary are the following:
+
+        ========   ===================================
+        Key        Value Description
+        ========   ===================================
+        label      tick label for the boxplot
+        mean       arithemetic mean value
+        med        50th percentile
+        q1         first quartile (25th percentile)
+        q3         third quartile (75th percentile)
+        cilo       lower notch around the median
+        ciho       upper notch around the median
+        whislo     end of the lower whisker
+        whishi     end of the upper whisker
+        fliers     outliers
+        ========   ===================================
+
+    Notes
+    -----
+    Non-bootstrapping approach to confidence interval uses Gaussian-based
+    asymptotic approximation:
+
+    .. math::
+
+        \mathrm{med} \pm 1.57 \\times \\frac{\mathrm{iqr}}{\sqrt{N}}
+
+    General approach from:
+    McGill, R., Tukey, J.W., and Larsen, W.A. (1978) "Variations of
+    Boxplots", The American Statistician, 32:12-16.
+
+    '''
+
+    def _bootstrap_median(data, N=5000):
+        # determine 95% confidence intervals of the median
+        M = len(data)
+        percentiles = [2.5, 97.5]
+
+        ii = np.random.randint(M, size=(N, M))
+        bsData = x[ii]
+        estimate = np.median(bsData, axis=1, overwrite_input=True)
+
+        CI = np.percentile(estimate, percentiles)
+        return CI
+
+    def _compute_conf_interval(data, med, iqr, bootstrap):
+        if bootstrap is not None:
+            # Do a bootstrap estimate of notch locations.
+            # get conf. intervals around median
+            CI = _bootstrap_median(data, N=bootstrap)
+            notch_min = CI[0]
+            notch_max = CI[1]
+        else:
+
+            N = len(data)
+            notch_min = med - 1.57 * iqr / np.sqrt(N)
+            notch_max = med + 1.57 * iqr / np.sqrt(N)
+
+        return notch_min, notch_max
+
+    # output is a list of dicts
+    bxpstats = []
+
+    # convert X to a list of lists
+    X = _reshape_2D(X)
+
+    ncols = len(X)
+    if labels is None:
+        labels = repeat(None)
+    elif len(labels) != ncols:
+        raise ValueError("Dimensions of labels and X must be compatible")
+
+    input_whis = whis
+    for ii, (x, label) in enumerate(zip(X, labels), start=0):
+
+        # empty dict
+        stats = {}
+        if label is not None:
+            stats['label'] = label
+
+        # restore whis to the input values in case it got changed in the loop
+        whis = input_whis
+
+        # note tricksyness, append up here and then mutate below
+        bxpstats.append(stats)
+
+        # if empty, bail
+        if len(x) == 0:
+            stats['fliers'] = np.array([])
+            stats['mean'] = np.nan
+            stats['med'] = np.nan
+            stats['q1'] = np.nan
+            stats['q3'] = np.nan
+            stats['cilo'] = np.nan
+            stats['ciho'] = np.nan
+            stats['whislo'] = np.nan
+            stats['whishi'] = np.nan
+            stats['med'] = np.nan
+            continue
+
+        # up-convert to an array, just to be safe
+        x = np.asarray(x)
+
+        # arithmetic mean
+        stats['mean'] = np.mean(x)
+
+        # medians and quartiles
+        q1, med, q3 = np.percentile(x, [25, 50, 75])
+
+        # interquartile range
+        stats['iqr'] = q3 - q1
+        if stats['iqr'] == 0:
+            whis = 'range'
+
+        # conf. interval around median
+        stats['cilo'], stats['cihi'] = _compute_conf_interval(
+            x, med, stats['iqr'], bootstrap
+        )
+
+        # lowest/highest non-outliers
+        if np.isscalar(whis):
+            if np.isreal(whis):
+                loval = q1 - whis * stats['iqr']
+                hival = q3 + whis * stats['iqr']
+            elif whis in ['range', 'limit', 'limits', 'min/max']:
+                loval = np.min(x)
+                hival = np.max(x)
+            else:
+                whismsg = ('whis must be a float, valid string, or '
+                           'list of percentiles')
+                raise ValueError(whismsg)
+        else:
+            loval = np.percentile(x, whis[0])
+            hival = np.percentile(x, whis[1])
+
+        # get high extreme
+        wiskhi = np.compress(x <= hival, x)
+        if len(wiskhi) == 0 or np.max(wiskhi) < q3:
+            stats['whishi'] = q3
+        else:
+            stats['whishi'] = np.max(wiskhi)
+
+        # get low extreme
+        wisklo = np.compress(x >= loval, x)
+        if len(wisklo) == 0 or np.min(wisklo) > q1:
+            stats['whislo'] = q1
+        else:
+            stats['whislo'] = np.min(wisklo)
+
+        # compute a single array of outliers
+        stats['fliers'] = np.hstack([
+            np.compress(x < stats['whislo'], x),
+            np.compress(x > stats['whishi'], x)
+        ])
+
+        # add in the remaining stats
+        stats['q1'], stats['med'], stats['q3'] = q1, med, q3
+
+
+    return bxpstats
+
+
+# FIXME I don't think this is used anywhere
+def unmasked_index_ranges(mask, compressed=True):
+    """
     Find index ranges where *mask* is *False*.
 
     *mask* will be flattened if it is not already 1-D.
@@ -1641,8 +2083,7 @@ def unmasked_index_ranges(mask, compressed=True):
 
     Prior to the transforms refactoring, this was used to support
     masked arrays in Line2D.
-
-    '''
+    """
     mask = mask.reshape(mask.size)
     m = np.concatenate(((1,), mask, (1,)))
     indices = np.arange(len(mask) + 1)
@@ -1670,79 +2111,6 @@ ls_mapper = dict(_linestyles)
 ls_mapper.update([(ls[1], ls[0]) for ls in _linestyles])
 
 
-def less_simple_linear_interpolation(x, y, xi, extrap=False):
-    """
-    This function has been moved to matplotlib.mlab -- please import
-    it from there
-    """
-    # deprecated from cbook in 0.98.4
-    warnings.warn('less_simple_linear_interpolation has been moved to '
-                  'matplotlib.mlab -- please import it from there',
-                  mplDeprecation)
-    import matplotlib.mlab as mlab
-    return mlab.less_simple_linear_interpolation(x, y, xi, extrap=extrap)
-
-
-def vector_lengths(X, P=2.0, axis=None):
-    """
-    This function has been moved to matplotlib.mlab -- please import
-    it from there
-    """
-    # deprecated from cbook in 0.98.4
-    warnings.warn('vector_lengths has been moved to matplotlib.mlab -- '
-                  'please import it from there', mplDeprecation)
-    import matplotlib.mlab as mlab
-    return mlab.vector_lengths(X, P=2.0, axis=axis)
-
-
-def distances_along_curve(X):
-    """
-    This function has been moved to matplotlib.mlab -- please import
-    it from there
-    """
-    # deprecated from cbook in 0.98.4
-    warnings.warn('distances_along_curve has been moved to matplotlib.mlab '
-                  '-- please import it from there', mplDeprecation)
-    import matplotlib.mlab as mlab
-    return mlab.distances_along_curve(X)
-
-
-def path_length(X):
-    """
-    This function has been moved to matplotlib.mlab -- please import
-    it from there
-    """
-    # deprecated from cbook in 0.98.4
-    warnings.warn('path_length has been moved to matplotlib.mlab '
-                  '-- please import it from there', mplDeprecation)
-    import matplotlib.mlab as mlab
-    return mlab.path_length(X)
-
-
-def is_closed_polygon(X):
-    """
-    This function has been moved to matplotlib.mlab -- please import
-    it from there
-    """
-    # deprecated from cbook in 0.98.4
-    warnings.warn('is_closed_polygon has been moved to matplotlib.mlab '
-                  '-- please import it from there', mplDeprecation)
-    import matplotlib.mlab as mlab
-    return mlab.is_closed_polygon(X)
-
-
-def quad2cubic(q0x, q0y, q1x, q1y, q2x, q2y):
-    """
-    This function has been moved to matplotlib.mlab -- please import
-    it from there
-    """
-    # deprecated from cbook in 0.98.4
-    warnings.warn('quad2cubic has been moved to matplotlib.mlab -- please '
-                  'import it from there', mplDeprecation)
-    import matplotlib.mlab as mlab
-    return mlab.quad2cubic(q0x, q0y, q1x, q1y, q2x, q2y)
-
-
 def align_iterators(func, *iterables):
     """
     This generator takes a bunch of iterables that are ordered by func
@@ -1791,7 +2159,7 @@ def is_math_text(s):
     # Did we find an even number of non-escaped dollar signs?
     # If so, treat is as math text.
     try:
-        s = unicode(s)
+        s = six.text_type(s)
     except UnicodeDecodeError:
         raise ValueError(
             "matplotlib display text must have all code points < 128 or use "
@@ -1803,6 +2171,114 @@ def is_math_text(s):
     return even_dollars
 
 
+def _reshape_2D(X):
+    """
+    Converts a non-empty list or an ndarray of two or fewer dimensions
+    into a list of iterable objects so that in
+
+        for v in _reshape_2D(X):
+
+    v is iterable and can be used to instantiate a 1D array.
+    """
+    if hasattr(X, 'shape'):
+        # one item
+        if len(X.shape) == 1:
+            if hasattr(X[0], 'shape'):
+                X = list(X)
+            else:
+                X = [X, ]
+
+        # several items
+        elif len(X.shape) == 2:
+            nrows, ncols = X.shape
+            if nrows == 1:
+                X = [X]
+            elif ncols == 1:
+                X = [X.ravel()]
+            else:
+                X = [X[:, i] for i in xrange(ncols)]
+        else:
+            raise ValueError("input `X` must have 2 or fewer dimensions")
+
+    if not hasattr(X[0], '__len__'):
+        X = [X]
+    else:
+        X = [np.ravel(x) for x in X]
+
+    return X
+
+
+def violin_stats(X, method, points=100):
+    '''
+    Returns a list of dictionaries of data which can be used to draw a series
+    of violin plots. See the `Returns` section below to view the required keys
+    of the dictionary. Users can skip this function and pass a user-defined set
+    of dictionaries to the `axes.vplot` method instead of using MPL to do the
+    calculations.
+
+    Parameters
+    ----------
+    X : array-like
+        Sample data that will be used to produce the gaussian kernel density
+        estimates. Must have 2 or fewer dimensions.
+
+    method : callable
+        The method used to calculate the kernel density estimate for each
+        column of data. When called via `method(v, coords)`, it should
+        return a vector of the values of the KDE evaluated at the values
+        specified in coords.
+
+    points : scalar, default = 100
+        Defines the number of points to evaluate each of the gaussian kernel
+        density estimates at.
+
+    Returns
+    -------
+
+    A list of dictionaries containing the results for each column of data.
+    The dictionaries contain at least the following:
+
+        - coords: A list of scalars containing the coordinates this particular
+          kernel density estimate was evaluated at.
+        - vals: A list of scalars containing the values of the kernel density
+          estimate at each of the coordinates given in `coords`.
+        - mean: The mean value for this column of data.
+        - median: The median value for this column of data.
+        - min: The minimum value for this column of data.
+        - max: The maximum value for this column of data.
+    '''
+
+    # List of dictionaries describing each of the violins.
+    vpstats = []
+
+    # Want X to be a list of data sequences
+    X = _reshape_2D(X)
+
+    for x in X:
+        # Dictionary of results for this distribution
+        stats = {}
+
+        # Calculate basic stats for the distribution
+        min_val = np.min(x)
+        max_val = np.max(x)
+
+        # Evaluate the kernel density estimate
+        coords = np.linspace(min_val, max_val, points)
+        stats['vals'] = method(x, coords)
+        stats['coords'] = coords
+
+        # Store additional statistics for this distribution
+        stats['mean'] = np.mean(x)
+        stats['median'] = np.median(x)
+        stats['min'] = min_val
+        stats['max'] = max_val
+
+        # Append to output
+        vpstats.append(stats)
+
+    return vpstats
+
+
 class _NestedClassGetter(object):
     # recipe from http://stackoverflow.com/a/11493777/741316
     """
@@ -1830,8 +2306,12 @@ class _InstanceMethodPickler(object):
     """
     def __init__(self, instancemethod):
         """Takes an instancemethod as its only argument."""
-        self.parent_obj = instancemethod.im_self
-        self.instancemethod_name = instancemethod.im_func.__name__
+        if six.PY3:
+            self.parent_obj = instancemethod.__self__
+            self.instancemethod_name = instancemethod.__func__.__name__
+        else:
+            self.parent_obj = instancemethod.im_self
+            self.instancemethod_name = instancemethod.im_func.__name__
 
     def get_instancemethod(self):
         return getattr(self.parent_obj, self.instancemethod_name)
@@ -1853,53 +2333,3 @@ except AttributeError:
 else:
     def _putmask(a, mask, values):
         return np.copyto(a, values, where=mask)
-
-
-def _check_output(*popenargs, **kwargs):
-    r"""Run command with arguments and return its output as a byte
-    string.
-
-    If the exit code was non-zero it raises a CalledProcessError.  The
-    CalledProcessError object will have the return code in the
-    returncode
-    attribute and output in the output attribute.
-
-    The arguments are the same as for the Popen constructor.  Example::
-
-    >>> check_output(["ls", "-l", "/dev/null"])
-    'crw-rw-rw- 1 root root 1, 3 Oct 18  2007 /dev/null\n'
-
-    The stdout argument is not allowed as it is used internally.
-    To capture standard error in the result, use stderr=STDOUT.::
-
-    >>> check_output(["/bin/sh", "-c",
-    ...               "ls -l non_existent_file ; exit 0"],
-    ...              stderr=STDOUT)
-    'ls: non_existent_file: No such file or directory\n'
-    """
-    if 'stdout' in kwargs:
-        raise ValueError('stdout argument not allowed, it will be overridden.')
-    process = subprocess.Popen(stdout=subprocess.PIPE, *popenargs, **kwargs)
-    output, unused_err = process.communicate()
-    retcode = process.poll()
-    if retcode:
-        cmd = kwargs.get("args")
-        if cmd is None:
-            cmd = popenargs[0]
-        raise subprocess.CalledProcessError(retcode, cmd, output=output)
-    return output
-
-
-# python2.7's subprocess provides a check_output method
-if hasattr(subprocess, 'check_output'):
-    check_output = subprocess.check_output
-else:
-    check_output = _check_output
-
-
-if __name__ == '__main__':
-    assert(allequal([1, 1, 1]))
-    assert(not allequal([1, 1, 0]))
-    assert(allequal([]))
-    assert(allequal(('a', 'a')))
-    assert(not allequal(('a', 'b')))
