@@ -1,12 +1,14 @@
 """
 An experimental module for reading dvi files output by TeX. Several
 limitations make this not (currently) useful as a general-purpose dvi
-preprocessor.
+preprocessor, but it is currently used by the pdf backend for
+processing usetex text.
 
 Interface::
 
   dvi = Dvi(filename, 72)
-  for page in dvi:          # iterate over pages
+  # iterate over pages (but only one page is supported for now):
+  for page in dvi:
       w, h, d = page.width, page.height, page.descent
       for x,y,font,glyph,width in page.text:
           fontname = font.texname
@@ -16,13 +18,25 @@ Interface::
           ...
 
 """
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
+import six
+from six.moves import xrange
 
 import errno
 import matplotlib
 import matplotlib.cbook as mpl_cbook
+from matplotlib.compat import subprocess
+from matplotlib import rcParams
 import numpy as np
 import struct
-import subprocess
+import sys
+import os
+
+if six.PY3:
+    def ord(x):
+        return x
 
 _dvistate = mpl_cbook.Bunch(pre=0, outer=1, inpage=2, post_post=3, finale=4)
 
@@ -44,12 +58,24 @@ class Dvi(object):
         self.dpi = dpi
         self.fonts = {}
         self.state = _dvistate.pre
+        self.baseline = self._get_baseline(filename)
+
+    def _get_baseline(self, filename):
+        if rcParams['text.latex.preview']:
+            base, ext = os.path.splitext(filename)
+            baseline_filename = base + ".baseline"
+            if os.path.exists(baseline_filename):
+                with open(baseline_filename, 'rb') as fd:
+                    l = fd.read().split()
+                height, depth, width = l
+                return float(depth)
+        return None
 
     def __iter__(self):
         """
         Iterate through the pages of the file.
 
-        Returns (text, pages) pairs, where:
+        Returns (text, boxes) pairs, where:
           text is a list of (x, y, fontnum, glyphnum, width) tuples
           boxes is a list of (x, y, height, width) tuples
 
@@ -86,8 +112,7 @@ class Dvi(object):
                 e = 0           # zero depth
             else:               # glyph
                 x,y,font,g,w = elt
-                h = _mul2012(font._scale, font._tfm.height[g])
-                e = _mul2012(font._scale, font._tfm.depth[g])
+                h,e = font._height_depth_of(g)
             minx = min(minx, x)
             miny = min(miny, y - h)
             maxx = max(maxx, x + w)
@@ -98,17 +123,22 @@ class Dvi(object):
             # special case for ease of debugging: output raw dvi coordinates
             return mpl_cbook.Bunch(text=self.text, boxes=self.boxes,
                                    width=maxx-minx, height=maxy_pure-miny,
-                                   descent=maxy-maxy_pure)
+                                   descent=descent)
 
         d = self.dpi / (72.27 * 2**16) # from TeX's "scaled points" to dpi units
-        text =  [ ((x-minx)*d, (maxy-y)*d, f, g, w*d)
+        if self.baseline is None:
+            descent = (maxy - maxy_pure) * d
+        else:
+            descent = self.baseline
+
+        text =  [ ((x-minx)*d, (maxy-y)*d - descent, f, g, w*d)
                   for (x,y,f,g,w) in self.text ]
-        boxes = [ ((x-minx)*d, (maxy-y)*d, h*d, w*d) for (x,y,h,w) in self.boxes ]
+        boxes = [ ((x-minx)*d, (maxy-y)*d - descent, h*d, w*d) for (x,y,h,w) in self.boxes ]
 
         return mpl_cbook.Bunch(text=text, boxes=boxes,
                                width=(maxx-minx)*d,
                                height=(maxy_pure-miny)*d,
-                               descent=(maxy-maxy_pure)*d)
+                               descent=descent)
 
     def _read(self):
         """
@@ -116,7 +146,7 @@ class Dvi(object):
         False if there were no more pages.
         """
         while True:
-            byte = ord(self.file.read(1))
+            byte = ord(self.file.read(1)[0])
             self._dispatch(byte)
 #             if self.state == _dvistate.inpage:
 #                 matplotlib.verbose.report(
@@ -131,8 +161,8 @@ class Dvi(object):
 
     def _arg(self, nbytes, signed=False):
         """
-        Read and return an integer argument "nbytes" long.
-        Signedness is determined by the "signed" keyword.
+        Read and return an integer argument *nbytes* long.
+        Signedness is determined by the *signed* keyword.
         """
         str = self.file.read(nbytes)
         value = ord(str[0])
@@ -144,7 +174,7 @@ class Dvi(object):
 
     def _dispatch(self, byte):
         """
-        Based on the opcode "byte", read the correct kinds of
+        Based on the opcode *byte*, read the correct kinds of
         arguments from the dvi file and call the method implementing
         that opcode with those arguments.
         """
@@ -213,41 +243,41 @@ class Dvi(object):
         elif byte == 248: self._post()
         elif byte == 249: self._post_post()
         else:
-            raise ValueError, "unknown command: byte %d"%byte
+            raise ValueError("unknown command: byte %d"%byte)
 
     def _pre(self, i, num, den, mag, comment):
         if self.state != _dvistate.pre:
-            raise ValueError, "pre command in middle of dvi file"
+            raise ValueError("pre command in middle of dvi file")
         if i != 2:
-            raise ValueError, "Unknown dvi format %d"%i
+            raise ValueError("Unknown dvi format %d"%i)
         if num != 25400000 or den != 7227 * 2**16:
-            raise ValueError, "nonstandard units in dvi file"
+            raise ValueError("nonstandard units in dvi file")
             # meaning: TeX always uses those exact values, so it
             # should be enough for us to support those
             # (There are 72.27 pt to an inch so 7227 pt =
             # 7227 * 2**16 sp to 100 in. The numerator is multiplied
             # by 10^5 to get units of 10**-7 meters.)
         if mag != 1000:
-            raise ValueError, "nonstandard magnification in dvi file"
+            raise ValueError("nonstandard magnification in dvi file")
             # meaning: LaTeX seems to frown on setting \mag, so
             # I think we can assume this is constant
         self.state = _dvistate.outer
 
     def _set_char(self, char):
         if self.state != _dvistate.inpage:
-            raise ValueError, "misplaced set_char in dvi file"
+            raise ValueError("misplaced set_char in dvi file")
         self._put_char(char)
         self.h += self.fonts[self.f]._width_of(char)
 
     def _set_rule(self, a, b):
         if self.state != _dvistate.inpage:
-            raise ValueError, "misplaced set_rule in dvi file"
+            raise ValueError("misplaced set_rule in dvi file")
         self._put_rule(a, b)
         self.h += b
 
     def _put_char(self, char):
         if self.state != _dvistate.inpage:
-            raise ValueError, "misplaced put_char in dvi file"
+            raise ValueError("misplaced put_char in dvi file")
         font = self.fonts[self.f]
         if font._vf is None:
             self.text.append((self.h, self.v, font, char,
@@ -270,7 +300,7 @@ class Dvi(object):
 
     def _put_rule(self, a, b):
         if self.state != _dvistate.inpage:
-            raise ValueError, "misplaced put_rule in dvi file"
+            raise ValueError("misplaced put_rule in dvi file")
         if a > 0 and b > 0:
             self.boxes.append((self.h, self.v, a, b))
 #             matplotlib.verbose.report(
@@ -282,8 +312,7 @@ class Dvi(object):
 
     def _bop(self, c0, c1, c2, c3, c4, c5, c6, c7, c8, c9, p):
         if self.state != _dvistate.outer:
-            raise ValueError, \
-                "misplaced bop in dvi file (state %d)" % self.state
+            raise ValueError("misplaced bop in dvi file (state %d)" % self.state)
         self.state = _dvistate.inpage
         self.h, self.v, self.w, self.x, self.y, self.z = 0, 0, 0, 0, 0, 0
         self.stack = []
@@ -292,87 +321,95 @@ class Dvi(object):
 
     def _eop(self):
         if self.state != _dvistate.inpage:
-            raise ValueError, "misplaced eop in dvi file"
+            raise ValueError("misplaced eop in dvi file")
         self.state = _dvistate.outer
         del self.h, self.v, self.w, self.x, self.y, self.z, self.stack
 
     def _push(self):
         if self.state != _dvistate.inpage:
-            raise ValueError, "misplaced push in dvi file"
+            raise ValueError("misplaced push in dvi file")
         self.stack.append((self.h, self.v, self.w, self.x, self.y, self.z))
 
     def _pop(self):
         if self.state != _dvistate.inpage:
-            raise ValueError, "misplaced pop in dvi file"
+            raise ValueError("misplaced pop in dvi file")
         self.h, self.v, self.w, self.x, self.y, self.z = self.stack.pop()
 
     def _right(self, b):
         if self.state != _dvistate.inpage:
-            raise ValueError, "misplaced right in dvi file"
+            raise ValueError("misplaced right in dvi file")
         self.h += b
 
     def _right_w(self, new_w):
         if self.state != _dvistate.inpage:
-            raise ValueError, "misplaced w in dvi file"
+            raise ValueError("misplaced w in dvi file")
         if new_w is not None:
             self.w = new_w
         self.h += self.w
 
     def _right_x(self, new_x):
         if self.state != _dvistate.inpage:
-            raise ValueError, "misplaced x in dvi file"
+            raise ValueError("misplaced x in dvi file")
         if new_x is not None:
             self.x = new_x
         self.h += self.x
 
     def _down(self, a):
         if self.state != _dvistate.inpage:
-            raise ValueError, "misplaced down in dvi file"
+            raise ValueError("misplaced down in dvi file")
         self.v += a
 
     def _down_y(self, new_y):
         if self.state != _dvistate.inpage:
-            raise ValueError, "misplaced y in dvi file"
+            raise ValueError("misplaced y in dvi file")
         if new_y is not None:
             self.y = new_y
         self.v += self.y
 
     def _down_z(self, new_z):
         if self.state != _dvistate.inpage:
-            raise ValueError, "misplaced z in dvi file"
+            raise ValueError("misplaced z in dvi file")
         if new_z is not None:
             self.z = new_z
         self.v += self.z
 
     def _fnt_num(self, k):
         if self.state != _dvistate.inpage:
-            raise ValueError, "misplaced fnt_num in dvi file"
+            raise ValueError("misplaced fnt_num in dvi file")
         self.f = k
 
     def _xxx(self, special):
-        matplotlib.verbose.report(
-            'Dvi._xxx: encountered special: %s'
-            % ''.join([(32 <= ord(ch) < 127) and ch
-                       or '<%02x>' % ord(ch)
-                       for ch in special]),
-            'debug')
+        if six.PY3:
+            matplotlib.verbose.report(
+                'Dvi._xxx: encountered special: %s'
+                % ''.join([(32 <= ord(ch) < 127) and chr(ch)
+                           or '<%02x>' % ord(ch)
+                           for ch in special]),
+                'debug')
+        else:
+            matplotlib.verbose.report(
+                'Dvi._xxx: encountered special: %s'
+                % ''.join([(32 <= ord(ch) < 127) and ch
+                           or '<%02x>' % ord(ch)
+                           for ch in special]),
+                'debug')
 
     def _fnt_def(self, k, c, s, d, a, l, n):
-        tfm = _tfmfile(n[-l:])
+        tfm = _tfmfile(n[-l:].decode('ascii'))
         if c != 0 and tfm.checksum != 0 and c != tfm.checksum:
-            raise ValueError, 'tfm checksum mismatch: %s'%n
+            raise ValueError('tfm checksum mismatch: %s'%n)
         # It seems that the assumption behind the following check is incorrect:
         #if d != tfm.design_size:
         #    raise ValueError, 'tfm design size mismatch: %d in dvi, %d in %s'%\
         #        (d, tfm.design_size, n)
 
-        vf = _vffile(n[-l:])
+        vf = _vffile(n[-l:].decode('ascii'))
 
         self.fonts[k] = DviFont(scale=s, tfm=tfm, texname=n, vf=vf)
 
     def _post(self):
         if self.state != _dvistate.outer:
-            raise ValueError, "misplaced post in dvi file"
+            raise ValueError("misplaced post in dvi file")
         self.state = _dvistate.post_post
         # TODO: actually read the postamble and finale?
         # currently post_post just triggers closing the file
@@ -385,22 +422,42 @@ class DviFont(object):
     Object that holds a font's texname and size, supports comparison,
     and knows the widths of glyphs in the same units as the AFM file.
     There are also internal attributes (for use by dviread.py) that
-    are _not_ used for comparison.
+    are *not* used for comparison.
 
     The size is in Adobe points (converted from TeX points).
+
+    .. attribute:: texname
+
+       Name of the font as used internally by TeX and friends. This
+       is usually very different from any external font names, and
+       :class:`dviread.PsfontsMap` can be used to find the external
+       name of the font.
+
+    .. attribute:: size
+
+       Size of the font in Adobe points, converted from the slightly
+       smaller TeX points.
+
+    .. attribute:: widths
+
+       Widths of glyphs in glyph-space units, typically 1/1000ths of
+       the point size.
+
     """
     __slots__ = ('texname', 'size', 'widths', '_scale', '_vf', '_tfm')
 
     def __init__(self, scale, tfm, texname, vf):
+        if six.PY3 and isinstance(texname, bytes):
+            texname = texname.decode('ascii')
         self._scale, self._tfm, self.texname, self._vf = \
             scale, tfm, texname, vf
         self.size = scale * (72.0 / (72.27 * 2**16))
         try:
-            nchars = max(tfm.width.iterkeys())
+            nchars = max(six.iterkeys(tfm.width)) + 1
         except ValueError:
             nchars = 0
         self.widths = [ (1000*tfm.width.get(char, 0)) >> 20
-                        for char in range(nchars) ]
+                        for char in xrange(nchars) ]
 
     def __eq__(self, other):
         return self.__class__ == other.__class__ and \
@@ -423,6 +480,24 @@ class DviFont(object):
             'debug')
         return 0
 
+    def _height_depth_of(self, char):
+        """
+        Height and depth of char in dvi units. For internal use by dviread.py.
+        """
+
+        result = []
+        for metric,name in ((self._tfm.height, "height"),
+                            (self._tfm.depth, "depth")):
+            value = metric.get(char, None)
+            if value is None:
+                matplotlib.verbose.report(
+                    'No %s for char %d in font %s' % (name, char, self.texname),
+                    'debug')
+                result.append(0)
+            else:
+                result.append(_mul2012(value, self._scale))
+        return result
+
 class Vf(Dvi):
     """
     A virtual font (\*.vf file) containing subroutines for dvi files.
@@ -436,11 +511,13 @@ class Vf(Dvi):
 
     def __init__(self, filename):
         Dvi.__init__(self, filename, 0)
-        self._first_font = None
-        self._chars = {}
-        self._packet_ends = None
-        self._read()
-        self.close()
+        try:
+            self._first_font = None
+            self._chars = {}
+            self._packet_ends = None
+            self._read()
+        finally:
+            self.close()
 
     def __getitem__(self, code):
         return self._chars[code]
@@ -453,10 +530,10 @@ class Vf(Dvi):
                 self._finalize_packet()
                 # fall through
             elif byte_at > self._packet_ends:
-                raise ValueError, "Packet length mismatch in vf file"
+                raise ValueError("Packet length mismatch in vf file")
             else:
                 if byte in (139, 140) or byte >= 243:
-                    raise ValueError, "Inappropriate opcode %d in vf file" % byte
+                    raise ValueError("Inappropriate opcode %d in vf file" % byte)
                 Dvi._dispatch(self, byte)
                 return
 
@@ -477,11 +554,11 @@ class Vf(Dvi):
         elif byte == 248:       # postamble (just some number of 248s)
             self.state = _dvistate.post_post
         else:
-            raise ValueError, "unknown vf opcode %d" % byte
+            raise ValueError("unknown vf opcode %d" % byte)
 
     def _init_packet(self, pl, cc, tfm):
         if self.state != _dvistate.outer:
-            raise ValueError, "Misplaced packet in vf file"
+            raise ValueError("Misplaced packet in vf file")
         self.state = _dvistate.inpage
         self._packet_ends = self.file.tell() + pl
         self._packet_char = cc
@@ -497,9 +574,9 @@ class Vf(Dvi):
 
     def _pre(self, i, x, cs, ds):
         if self.state != _dvistate.pre:
-            raise ValueError, "pre command in middle of vf file"
+            raise ValueError("pre command in middle of vf file")
         if i != 202:
-            raise ValueError, "Unknown vf format %d" % i
+            raise ValueError("Unknown vf format %d" % i)
         if len(x):
             matplotlib.verbose.report('vf file comment: ' + x, 'debug')
         self.state = _dvistate.outer
@@ -532,56 +609,73 @@ class Tfm(object):
     A TeX Font Metric file. This implementation covers only the bare
     minimum needed by the Dvi class.
 
-    Attributes:
+    .. attribute:: checksum
 
-      checksum: for verifying against dvi file
+       Used for verifying against the dvi file.
 
-      design_size: design size of the font (in what units?)
+    .. attribute:: design_size
 
-      width[i]: width of character \#i, needs to be scaled
-        by the factor specified in the dvi file
-        (this is a dict because indexing may not start from 0)
+       Design size of the font (in what units?)
 
-      height[i], depth[i]: height and depth of character \#i
+    .. attribute::  width
+
+       Width of each character, needs to be scaled by the factor
+       specified in the dvi file. This is a dict because indexing may
+       not start from 0.
+
+    .. attribute:: height
+
+       Height of each character.
+
+    .. attribute:: depth
+
+       Depth of each character.
     """
     __slots__ = ('checksum', 'design_size', 'width', 'height', 'depth')
 
     def __init__(self, filename):
         matplotlib.verbose.report('opening tfm file ' + filename, 'debug')
-        file = open(filename, 'rb')
-
-        try:
+        with open(filename, 'rb') as file:
             header1 = file.read(24)
             lh, bc, ec, nw, nh, nd = \
-                struct.unpack('!6H', header1[2:14])
+                struct.unpack(str('!6H'), header1[2:14])
             matplotlib.verbose.report(
                 'lh=%d, bc=%d, ec=%d, nw=%d, nh=%d, nd=%d' % (
                     lh, bc, ec, nw, nh, nd), 'debug')
             header2 = file.read(4*lh)
             self.checksum, self.design_size = \
-                struct.unpack('!2I', header2[:8])
+                struct.unpack(str('!2I'), header2[:8])
             # there is also encoding information etc.
             char_info = file.read(4*(ec-bc+1))
             widths = file.read(4*nw)
             heights = file.read(4*nh)
             depths = file.read(4*nd)
-        finally:
-            file.close()
 
         self.width, self.height, self.depth = {}, {}, {}
         widths, heights, depths = \
-            [ struct.unpack('!%dI' % (len(x)/4), x)
+            [ struct.unpack(str('!%dI') % (len(x)/4), x)
               for x in (widths, heights, depths) ]
-        for i in range(ec-bc):
-            self.width[bc+i] = _fix2comp(widths[ord(char_info[4*i])])
-            self.height[bc+i] = _fix2comp(heights[ord(char_info[4*i+1]) >> 4])
-            self.depth[bc+i] = _fix2comp(depths[ord(char_info[4*i+1]) & 0xf])
-
+        for idx, char in enumerate(xrange(bc, ec+1)):
+            self.width[char] = _fix2comp(widths[ord(char_info[4*idx])])
+            self.height[char] = _fix2comp(heights[ord(char_info[4*idx+1]) >> 4])
+            self.depth[char] = _fix2comp(depths[ord(char_info[4*idx+1]) & 0xf])
 
 class PsfontsMap(object):
     """
     A psfonts.map formatted file, mapping TeX fonts to PS fonts.
-    Usage: map = PsfontsMap('.../psfonts.map'); map['cmr10']
+    Usage::
+
+     >>> map = PsfontsMap(find_tex_file('pdftex.map'))
+     >>> entry = map['ptmbo8r']
+     >>> entry.texname
+     'ptmbo8r'
+     >>> entry.psname
+     'Times-Bold'
+     >>> entry.encoding
+     '/usr/local/texlive/2008/texmf-dist/fonts/enc/dvips/base/8r.enc'
+     >>> entry.effects
+     {'slant': 0.16700000000000001}
+     >>> entry.filename
 
     For historical reasons, TeX knows many Type-1 fonts by different
     names than the outside world. (For one thing, the names have to
@@ -594,24 +688,25 @@ class PsfontsMap(object):
     file names.
 
     A texmf tree typically includes mapping files called e.g.
-    psfonts.map, pdftex.map,  dvipdfm.map. psfonts.map is used by
+    psfonts.map, pdftex.map, dvipdfm.map. psfonts.map is used by
     dvips, pdftex.map by pdfTeX, and dvipdfm.map by dvipdfm.
-    psfonts.map might avoid embedding the 35 PostScript fonts, while
-    the pdf-related files perhaps only avoid the "Base 14" pdf fonts.
-    But the user may have configured these files differently.
+    psfonts.map might avoid embedding the 35 PostScript fonts (i.e.,
+    have no filename for them, as in the Times-Bold example above),
+    while the pdf-related files perhaps only avoid the "Base 14" pdf
+    fonts. But the user may have configured these files differently.
     """
     __slots__ = ('_font',)
 
     def __init__(self, filename):
         self._font = {}
-        file = open(filename, 'rt')
-        try:
+        with open(filename, 'rt') as file:
             self._parse(file)
-        finally:
-            file.close()
 
     def __getitem__(self, texname):
-        result = self._font[texname]
+        try:
+            result = self._font[texname]
+        except KeyError:
+            result = self._font[texname.decode('ascii')]
         fn, enc = result.filename, result.encoding
         if fn is not None and not fn.startswith('/'):
             result.filename = find_tex_file(fn)
@@ -654,22 +749,44 @@ class PsfontsMap(object):
         There is some difference between <foo.pfb and <<bar.pfb in
         subsetting, but I have no example of << in my TeX installation.
         """
+
+        # If the map file specifies multiple encodings for a font, we
+        # follow pdfTeX in choosing the last one specified. Such
+        # entries are probably mistakes but they have occurred.
+        # http://tex.stackexchange.com/questions/10826/
+        # http://article.gmane.org/gmane.comp.tex.pdftex/4914
+
         texname, psname = words[:2]
-        effects, encoding, filename = [], None, None
+        effects, encoding, filename = '', None, None
         for word in words[2:]:
             if not word.startswith('<'):
-                effects.append(word)
+                effects = word
             else:
                 word = word.lstrip('<')
-                if word.startswith('['):
-                    assert encoding is None
-                    encoding = word[1:]
-                elif word.endswith('.enc'):
-                    assert encoding is None
-                    encoding = word
+                if word.startswith('[') or word.endswith('.enc'):
+                    if encoding is not None:
+                        matplotlib.verbose.report(
+                            'Multiple encodings for %s = %s'
+                            % (texname, psname), 'debug')
+                    if word.startswith('['):
+                        encoding = word[1:]
+                    else:
+                        encoding = word
                 else:
                     assert filename is None
                     filename = word
+
+        eff = effects.split()
+        effects = {}
+        try:
+            effects['slant'] = float(eff[eff.index('SlantFont')-1])
+        except ValueError:
+            pass
+        try:
+            effects['extend'] = float(eff[eff.index('ExtendFont')-1])
+        except ValueError:
+            pass
+
         self._font[texname] = mpl_cbook.Bunch(
             texname=texname, psname=psname, effects=effects,
             encoding=encoding, filename=filename)
@@ -688,13 +805,10 @@ class Encoding(object):
     __slots__ = ('encoding',)
 
     def __init__(self, filename):
-        file = open(filename, 'rt')
-        try:
+        with open(filename, 'rt') as file:
             matplotlib.verbose.report('Parsing TeX encoding ' + filename, 'debug-annoying')
             self.encoding = self._parse(file)
-            matplotlib.verbose.report('Result: ' + `self.encoding`, 'debug-annoying')
-        finally:
-            file.close()
+            matplotlib.verbose.report('Result: ' + repr(self.encoding), 'debug-annoying')
 
     def __iter__(self):
         for name in self.encoding:
@@ -727,46 +841,45 @@ class Encoding(object):
                         subwords = w.split('/')
                         result.extend(subwords[1:])
                     else:
-                        raise ValueError, "Broken name in encoding file: " + w
+                        raise ValueError("Broken name in encoding file: " + w)
 
         return result
 
 def find_tex_file(filename, format=None):
     """
-    Call kpsewhich to find a file in the texmf tree.
-    If format is not None, it is used as the value for the --format option.
-    See the kpathsea documentation for more information.
+    Call :program:`kpsewhich` to find a file in the texmf tree. If
+    *format* is not None, it is used as the value for the
+    :option:`--format` option.
 
     Apparently most existing TeX distributions on Unix-like systems
     use kpathsea. I hear MikTeX (a popular distribution on Windows)
     doesn't use kpathsea, so what do we do? (TODO)
+
+    .. seealso::
+
+      `Kpathsea documentation <http://www.tug.org/kpathsea/>`_
+        The library that :program:`kpsewhich` is part of.
     """
 
     cmd = ['kpsewhich']
     if format is not None:
         cmd += ['--format=' + format]
     cmd += [filename]
-    
+
     matplotlib.verbose.report('find_tex_file(%s): %s' \
                                   % (filename,cmd), 'debug')
-    pipe = subprocess.Popen(cmd, stdout=subprocess.PIPE)
+    # stderr is unused, but reading it avoids a subprocess optimization
+    # that breaks EINTR handling in some Python versions:
+    # http://bugs.python.org/issue12493
+    # https://github.com/matplotlib/matplotlib/issues/633
+    pipe = subprocess.Popen(cmd, stdout=subprocess.PIPE,
+                            stderr=subprocess.PIPE)
     result = pipe.communicate()[0].rstrip()
     matplotlib.verbose.report('find_tex_file result: %s' % result,
                               'debug')
-    return result
+    return result.decode('ascii')
 
-def _read_nointr(pipe, bufsize=-1):
-    while True:
-        try:
-            return pipe.read(bufsize)
-        except OSError, e:
-            if e.errno == errno.EINTR:
-                continue
-            else:
-                raise
-        
-
-# With multiple text objects per figure (e.g. tick labels) we may end
+# With multiple text objects per figure (e.g., tick labels) we may end
 # up reading the same tfm and vf files many times, so we implement a
 # simple cache. TODO: is this worth making persistent?
 
@@ -805,12 +918,12 @@ if __name__ == '__main__':
     dvi = Dvi(fname, dpi)
     fontmap = PsfontsMap(find_tex_file('pdftex.map'))
     for page in dvi:
-        print '=== new page ==='
+        print('=== new page ===')
         fPrev = None
         for x,y,f,c,w in page.text:
             if f != fPrev:
-                print 'font', f.texname, 'scaled', f._scale/pow(2.0,20)
+                print('font', f.texname, 'scaled', f._scale/pow(2.0,20))
                 fPrev = f
-            print x,y,c, 32 <= c < 128 and chr(c) or '.', w
+            print(x,y,c, 32 <= c < 128 and chr(c) or '.', w)
         for x,y,w,h in page.boxes:
-            print x,y,'BOX',w,h
+            print(x,y,'BOX',w,h)
