@@ -14,39 +14,45 @@ Text instance. The width and height of the TextArea instance is the
 width and height of the its child text.
 """
 
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
 
-from __future__ import print_function
+import six
+from six.moves import xrange, zip
+
+import warnings
 import matplotlib.transforms as mtransforms
 import matplotlib.artist as martist
 import matplotlib.text as mtext
 import numpy as np
-from matplotlib.transforms import Bbox, BboxBase, TransformedBbox, \
-     IdentityTransform, BboxTransformFrom
+from matplotlib.transforms import Bbox, BboxBase, TransformedBbox
 
 from matplotlib.font_manager import FontProperties
 from matplotlib.patches import FancyBboxPatch, FancyArrowPatch
 from matplotlib import rcParams
 
 from matplotlib import docstring
-import matplotlib.cbook as cbook
 
 #from bboximage import BboxImage
 from matplotlib.image import BboxImage
 
 from matplotlib.patches import bbox_artist as mbbox_artist
+from matplotlib.text import _AnnotationBase
 
 
 DEBUG = False
+
+
 # for debuging use
 def bbox_artist(*args, **kwargs):
     if DEBUG:
         mbbox_artist(*args, **kwargs)
 
-
 # _get_packed_offsets() and _get_aligned_offsets() are coded assuming
 # that we are packing boxes horizontally. But same function will be
 # used with vertical packing.
 
+
 def _get_packed_offsets(wd_list, total, sep, mode="fixed"):
     """
     Geiven a list of (width, xdescent) of each boxes, calculate the
@@ -60,7 +66,7 @@ def _get_packed_offsets(wd_list, total, sep, mode="fixed"):
     *mode* : packing mode. 'fixed', 'expand', or 'equal'.
     """
 
-    w_list, d_list = zip(*wd_list)
+    w_list, d_list = list(zip(*wd_list))
     # d_list is currently not used.
 
     if mode == "fixed":
@@ -73,7 +79,10 @@ def _get_packed_offsets(wd_list, total, sep, mode="fixed"):
         return total, offsets
 
     elif mode == "expand":
-        sep = (total - sum(w_list)) / (len(w_list) - 1.)
+        if len(w_list) > 1:
+            sep = (total - sum(w_list)) / (len(w_list) - 1.)
+        else:
+            sep = 0.
         offsets_ = np.add.accumulate([0] + [w + sep for w in w_list])
         offsets = offsets_[:-1]
 
@@ -140,27 +149,32 @@ class OffsetBox(martist.Artist):
 
         super(OffsetBox, self).__init__(*args, **kwargs)
 
+        # Clipping has not been implemented in the OffesetBox family, so
+        # disable the clip flag for consistency. It can always be turned back
+        # on to zero effect.
+        self.set_clip_on(False)
+
         self._children = []
         self._offset = (0, 0)
 
     def __getstate__(self):
         state = martist.Artist.__getstate__(self)
-        
+
         # pickle cannot save instancemethods, so handle them here
-        from cbook import _InstanceMethodPickler
+        from .cbook import _InstanceMethodPickler
         import inspect
-        
+
         offset = state['_offset']
         if inspect.ismethod(offset):
             state['_offset'] = _InstanceMethodPickler(offset)
         return state
-        
+
     def __setstate__(self, state):
         self.__dict__ = state
-        from cbook import _InstanceMethodPickler
+        from .cbook import _InstanceMethodPickler
         if isinstance(self._offset, _InstanceMethodPickler):
             self._offset = self._offset.get_instancemethod()
-        
+
     def set_figure(self, fig):
         """
         Set the figure
@@ -192,7 +206,7 @@ class OffsetBox(martist.Artist):
 
         accepts extent of the box
         """
-        if callable(self._offset):
+        if six.callable(self._offset):
             return self._offset(width, height, xdescent, ydescent, renderer)
         else:
             return self._offset
@@ -249,7 +263,8 @@ class OffsetBox(martist.Artist):
         to the given *renderer*.
         """
 
-        width, height, xdescent, ydescent, offsets = self.get_extent_offsets(renderer)
+        width, height, xdescent, ydescent, offsets = self.get_extent_offsets(
+                                                        renderer)
 
         px, py = self.get_offset(width, height, xdescent, ydescent, renderer)
 
@@ -265,18 +280,32 @@ class PackerBase(OffsetBox):
                  align=None, mode=None,
                  children=None):
         """
-        *pad* : boundary pad
-        *sep* : spacing between items
-        *width*, *height* : width and height of the container box.
-           calculated if None.
-        *align* : alignment of boxes. Can be one of 'top', 'bottom',
-           'left', 'right', 'center' and 'baseline'
-        *mode* : packing mode
+        Parameters
+        ----------
+        pad : float, optional
+            Boundary pad.
 
-        .. note::
-          *pad* and *sep* need to given in points and will be
-          scale with the renderer dpi, while *width* and *height*
-          need to be in pixels.
+        sep : float, optional
+            Spacing between items.
+
+        width : float, optional
+
+        height : float, optional
+           Width and height of the container box, calculated if
+           `None`.
+
+        align : str, optional
+            Alignment of boxes. Can be one of ``top``, ``bottom``,
+            ``left``, ``right``, ``center`` and ``baseline``
+
+        mode : str, optional
+            Packing mode.
+
+        Notes
+        -----
+        *pad* and *sep* need to given in points and will be scale with
+        the renderer dpi, while *width* and *height* need to be in
+        pixels.
         """
         super(PackerBase, self).__init__()
 
@@ -299,24 +328,37 @@ class VPacker(PackerBase):
                  align="baseline", mode="fixed",
                  children=None):
         """
-        *pad* : boundary pad
-        *sep* : spacing between items
-        *width*, *height* : width and height of the container box.
-           calculated if None.
-        *align* : alignment of boxes
-        *mode* : packing mode
+        Parameters
+        ----------
+        pad : float, optional
+            Boundary pad.
 
-        .. note::
-          *pad* and *sep* need to given in points and will be
-          scale with the renderer dpi, while *width* and *height*
-          need to be in pixels.
+        sep : float, optional
+            Spacing between items.
+
+        width : float, optional
+
+        height : float, optional
+
+            width and height of the container box, calculated if
+            `None`.
+
+        align : str, optional
+            Alignment of boxes.
+
+        mode : str, optional
+            Packing mode.
+
+        Notes
+        -----
+        *pad* and *sep* need to given in points and will be scale with
+        the renderer dpi, while *width* and *height* need to be in
+        pixels.
         """
         super(VPacker, self).__init__(pad, sep, width, height,
                                       align, mode,
                                       children)
 
-
-
     def get_extent_offsets(self, renderer):
         """
         update offset of childrens and return the extents of the box
@@ -331,10 +373,10 @@ class VPacker(PackerBase):
                 if isinstance(c, PackerBase) and c.mode == "expand":
                     c.set_width(self.width)
 
-        whd_list = [c.get_extent(renderer) for c in self.get_visible_children()]
+        whd_list = [c.get_extent(renderer)
+                    for c in self.get_visible_children()]
         whd_list = [(w, h, xd, (h - yd)) for w, h, xd, yd in whd_list]
 
-
         wd_list = [(w, xd) for w, h, xd, yd in whd_list]
         width, xdescent, xoffsets = _get_aligned_offsets(wd_list,
                                                          self.width,
@@ -351,10 +393,9 @@ class VPacker(PackerBase):
         #w, h, xd, h_yd = whd_list[-1]
         yoffsets = yoffsets - ydescent
 
-
         return width + 2 * pad, height + 2 * pad, \
                xdescent + pad, ydescent + pad, \
-               zip(xoffsets, yoffsets)
+               list(zip(xoffsets, yoffsets))
 
 
 class HPacker(PackerBase):
@@ -366,17 +407,31 @@ class HPacker(PackerBase):
                  align="baseline", mode="fixed",
                  children=None):
         """
-        *pad* : boundary pad
-        *sep* : spacing between items
-        *width*, *height* : width and height of the container box.
-           calculated if None.
-        *align* : alignment of boxes
-        *mode* : packing mode
+        Parameters
+        ----------
+        pad : float, optional
+            Boundary pad.
 
-        .. note::
-          *pad* and *sep* need to given in points and will be
-          scale with the renderer dpi, while *width* and *height*
-          need to be in pixels.
+        sep : float, optional
+            Spacing between items.
+
+        width : float, optional
+
+        height : float, optional
+           Width and height of the container box, calculated if
+           `None`.
+
+        align : str
+           Alignment of boxes.
+
+        mode : str
+           Packing mode.
+
+        Notes
+        -----
+        *pad* and *sep* need to given in points and will be scale with
+        the renderer dpi, while *width* and *height* need to be in
+        pixels.
         """
         super(HPacker, self).__init__(pad, sep, width, height,
                                       align, mode, children)
@@ -389,7 +444,8 @@ class HPacker(PackerBase):
         pad = self.pad * dpicor
         sep = self.sep * dpicor
 
-        whd_list = [c.get_extent(renderer) for c in self.get_visible_children()]
+        whd_list = [c.get_extent(renderer)
+                    for c in self.get_visible_children()]
 
         if not whd_list:
             return 2 * pad, 2 * pad, pad, pad, []
@@ -399,7 +455,7 @@ class HPacker(PackerBase):
             ydescent = max([yd for w, h, xd, yd in whd_list])
             height = height_descent + ydescent
         else:
-            height = self.height - 2 * pad # width w/o pad
+            height = self.height - 2 * pad  # width w/o pad
 
         hd_list = [(h, yd) for w, h, xd, yd in whd_list]
         height, ydescent, yoffsets = _get_aligned_offsets(hd_list,
@@ -407,7 +463,7 @@ class HPacker(PackerBase):
                                                           self.align)
 
         pack_list = [(w, xd) for w, h, xd, yd in whd_list]
-        
+
         width, xoffsets_ = _get_packed_offsets(pack_list, self.width,
                                                sep, self.mode)
 
@@ -418,7 +474,7 @@ class HPacker(PackerBase):
 
         return width + 2 * pad, height + 2 * pad, \
                xdescent + pad, ydescent + pad, \
-               zip(xoffsets, yoffsets)
+               list(zip(xoffsets, yoffsets))
 
 
 class PaddedBox(OffsetBox):
@@ -440,7 +496,7 @@ class PaddedBox(OffsetBox):
         self.patch = FancyBboxPatch(
             xy=(0.0, 0.0), width=1., height=1.,
             facecolor='w', edgecolor='k',
-            mutation_scale=1, #self.prop.get_size_in_points(),
+            mutation_scale=1,  # self.prop.get_size_in_points(),
             snap=True
             )
 
@@ -471,7 +527,8 @@ class PaddedBox(OffsetBox):
         to the given *renderer*.
         """
 
-        width, height, xdescent, ydescent, offsets = self.get_extent_offsets(renderer)
+        width, height, xdescent, ydescent, offsets = self.get_extent_offsets(
+                                                        renderer)
 
         px, py = self.get_offset(width, height, xdescent, ydescent, renderer)
 
@@ -563,7 +620,7 @@ class DrawingArea(OffsetBox):
         get the bounding box in display space.
         '''
         w, h, xd, yd = self.get_extent(renderer)
-        ox, oy = self.get_offset() #w, h, xd, yd)
+        ox, oy = self.get_offset()  # w, h, xd, yd)
 
         return mtransforms.Bbox.from_bounds(ox - xd, oy - yd, w, h)
 
@@ -610,12 +667,20 @@ class TextArea(OffsetBox):
                  minimumdescent=True,
                  ):
         """
-        *s* : a string to be displayed.
-        *textprops* : property dictionary for the text
-        *multilinebaseline* : If True, baseline for multiline text is
-                              adjusted so that it is (approximatedly)
-                              center-aligned with singleline text.
-        *minimumdescent*  : If True, the box has a minimum descent of "p".
+        Parameters
+        ----------
+        s : str
+            a string to be displayed.
+
+        textprops : `~matplotlib.font_manager.FontProperties`, optional
+
+        multilinebaseline : bool, optional
+            If `True`, baseline for multiline text is adjusted so that
+            it is (approximatedly) center-aligned with singleline
+            text.
+
+        minimumdescent : bool, optional
+            If `True`, the box has a minimum descent of "p".
         """
         if textprops is None:
             textprops = {}
@@ -629,12 +694,12 @@ class TextArea(OffsetBox):
 
         self._children = [self._text]
 
-
         self.offset_transform = mtransforms.Affine2D()
         self.offset_transform.clear()
         self.offset_transform.translate(0, 0)
         self._baseline_transform = mtransforms.Affine2D()
-        self._text.set_transform(self.offset_transform + self._baseline_transform)
+        self._text.set_transform(self.offset_transform +
+                                 self._baseline_transform)
 
         self._multilinebaseline = multilinebaseline
         self._minimumdescent = minimumdescent
@@ -706,7 +771,7 @@ class TextArea(OffsetBox):
         get the bounding box in display space.
         '''
         w, h, xd, yd = self.get_extent(renderer)
-        ox, oy = self.get_offset() #w, h, xd, yd)
+        ox, oy = self.get_offset()  # w, h, xd, yd)
         return mtransforms.Bbox.from_bounds(ox - xd, oy - yd, w, h)
 
     def get_extent(self, renderer):
@@ -714,13 +779,10 @@ class TextArea(OffsetBox):
         _, h_, d_ = renderer.get_text_width_height_descent(
             "lp", self._text._fontproperties, ismath=False)
 
-        bbox, info = self._text._get_layout(renderer)
+        bbox, info, d = self._text._get_layout(renderer)
         w, h = bbox.width, bbox.height
 
-        line = info[-1][0] # last line
-        _, hh, dd = renderer.get_text_width_height_descent(
-            line, self._text._fontproperties, ismath=ismath)
-        d = dd  # the baseline of the last line
+        line = info[-1][0]  # last line
 
         self._baseline_transform.clear()
 
@@ -729,7 +791,7 @@ class TextArea(OffsetBox):
             self._baseline_transform.translate(0, d - d_new)
             d = d_new
 
-        else: # single line
+        else:  # single line
 
             h_d = max(h_ - d_, h - d)
 
@@ -824,13 +886,13 @@ class AuxTransformBox(OffsetBox):
         get the bounding box in display space.
         '''
         w, h, xd, yd = self.get_extent(renderer)
-        ox, oy = self.get_offset() #w, h, xd, yd)
+        ox, oy = self.get_offset()  # w, h, xd, yd)
         return mtransforms.Bbox.from_bounds(ox - xd, oy - yd, w, h)
 
     def get_extent(self, renderer):
 
         # clear the offset transforms
-        _off = self.offset_transform.to_values() # to be restored later
+        _off = self.offset_transform.to_values()  # to be restored later
         self.ref_offset_transform.clear()
         self.offset_transform.clear()
 
@@ -866,7 +928,7 @@ class AnchoredOffsetbox(OffsetBox):
     the offset box is anchored against its parent axes. You may
     explicitly specify the bbox_to_anchor.
     """
-    zorder = 5 # zorder of the legend
+    zorder = 5  # zorder of the legend
 
     def __init__(self, loc,
                  pad=0.4, borderpad=0.5,
@@ -1008,7 +1070,8 @@ class AnchoredOffsetbox(OffsetBox):
         renderer (because of the padding).
         """
         if fontsize is None:
-            fontsize = renderer.points_to_pixels(self.prop.get_size_in_points())
+            fontsize = renderer.points_to_pixels(
+                            self.prop.get_size_in_points())
 
         def _offset(w, h, xd, yd, renderer, fontsize=fontsize, self=self):
             bbox = Bbox.from_bounds(0, 0, w, h)
@@ -1033,7 +1096,8 @@ class AnchoredOffsetbox(OffsetBox):
     def draw(self, renderer):
         "draw the artist"
 
-        if not self.get_visible(): return
+        if not self.get_visible():
+            return
 
         fontsize = renderer.points_to_pixels(self.prop.get_size_in_points())
         self._update_offset_func(renderer, fontsize)
@@ -1044,7 +1108,6 @@ class AnchoredOffsetbox(OffsetBox):
             self.update_frame(bbox, fontsize)
             self.patch.draw(renderer)
 
-
         width, height, xdescent, ydescent = self.get_extent(renderer)
 
         px, py = self.get_offset(width, height, xdescent, ydescent, renderer)
@@ -1057,20 +1120,20 @@ class AnchoredOffsetbox(OffsetBox):
         return the position of the bbox anchored at the parentbbox
         with the loc code, with the borderpad.
         """
-        assert loc in range(1, 11) # called only internally
+        assert loc in range(1, 11)  # called only internally
 
-        BEST, UR, UL, LL, LR, R, CL, CR, LC, UC, C = range(11)
+        BEST, UR, UL, LL, LR, R, CL, CR, LC, UC, C = list(xrange(11))
 
-        anchor_coefs = {UR:"NE",
-                      UL:"NW",
-                      LL:"SW",
-                      LR:"SE",
-                      R:"E",
-                      CL:"W",
-                      CR:"E",
-                      LC:"S",
-                      UC:"N",
-                      C:"C"}
+        anchor_coefs = {UR: "NE",
+                        UL: "NW",
+                        LL: "SW",
+                        LR: "SE",
+                        R: "E",
+                        CL: "W",
+                        CR: "E",
+                        LC: "S",
+                        UC: "N",
+                        C: "C"}
 
         c = anchor_coefs[loc]
 
@@ -1081,20 +1144,43 @@ class AnchoredOffsetbox(OffsetBox):
 
 class AnchoredText(AnchoredOffsetbox):
     """
-    AnchoredOffsetbox with Text
+    AnchoredOffsetbox with Text.
     """
 
     def __init__(self, s, loc, pad=0.4, borderpad=0.5, prop=None, **kwargs):
         """
-        *s* : string
-        *loc* : location code
-        *prop* : font property
-        *pad* : pad between the text and the frame as fraction of the font size.
-        *borderpad* : pad between the frame and the axes (or bbox_to_anchor).
+        Parameters
+        ----------
+        s : string
+            Text.
 
-        other keyword parameters of AnchoredOffsetbox are also allowed.
+        loc : str
+            Location code.
+
+        pad : float, optional
+            Pad between the text and the frame as fraction of the font
+            size.
+
+        borderpad : float, optional
+            Pad between the frame and the axes (or *bbox_to_anchor*).
+
+        prop : `matplotlib.font_manager.FontProperties`
+            Font properties.
+
+        Notes
+        -----
+        Other keyword parameters of `AnchoredOffsetbox` are also
+        allowed.
         """
 
+        if prop is None:
+            prop = {}
+        propkeys = list(six.iterkeys(prop))
+        badkwargs = ('ha', 'horizontalalignment', 'va', 'verticalalignment')
+        if set(badkwargs) & set(propkeys):
+            warnings.warn("Mixing horizontalalignment or verticalalignment "
+                    "with AnchoredText is not supported.")
+
         self.txt = TextArea(s, textprops=prop,
                             minimumdescent=False)
         fp = self.txt._text.get_fontproperties()
@@ -1186,7 +1272,8 @@ class OffsetImage(OffsetBox):
 
     def get_extent(self, renderer):
 
-        if self._dpi_cor: # True, do correction
+        # FIXME dpi_cor is never used
+        if self._dpi_cor:  # True, do correction
             dpi_cor = renderer.points_to_pixels(1.)
         else:
             dpi_cor = 1.
@@ -1206,9 +1293,6 @@ class OffsetImage(OffsetBox):
         #bbox_artist(self, renderer, fill=False, props=dict(pad=0.))
 
 
-from matplotlib.text import _AnnotationBase
-
-
 class AnnotationBbox(martist.Artist, _AnnotationBase):
     """
     Annotation-like class, but with offsetbox instead of Text.
@@ -1217,12 +1301,13 @@ class AnnotationBbox(martist.Artist, _AnnotationBase):
 
     def __str__(self):
         return "AnnotationBbox(%g,%g)" % (self.xy[0], self.xy[1])
+
     @docstring.dedent_interpd
     def __init__(self, offsetbox, xy,
                  xybox=None,
                  xycoords='data',
                  boxcoords=None,
-                 frameon=True, pad=0.4, # BboxPatch
+                 frameon=True, pad=0.4,  # BboxPatch
                  annotation_clip=None,
                  box_alignment=(0.5, 0.5),
                  bboxprops=None,
@@ -1251,6 +1336,15 @@ class AnnotationBbox(martist.Artist, _AnnotationBase):
 
         self.set_fontsize(fontsize)
 
+        if xybox is None:
+            self.xybox = xy
+        else:
+            self.xybox = xybox
+
+        if boxcoords is None:
+            self.boxcoords = xycoords
+        else:
+            self.boxcoords = boxcoords
 
         if arrowprops is not None:
             self._arrow_relpos = self.arrowprops.pop("relpos", (0.5, 0.5))
@@ -1261,8 +1355,8 @@ class AnnotationBbox(martist.Artist, _AnnotationBase):
             self.arrow_patch = None
 
         _AnnotationBase.__init__(self,
-                                 xy, xytext=xybox,
-                                 xycoords=xycoords, textcoords=boxcoords,
+                                 xy,
+                                 xycoords=xycoords,
                                  annotation_clip=annotation_clip)
 
         martist.Artist.__init__(self, **kwargs)
@@ -1282,6 +1376,22 @@ class AnnotationBbox(martist.Artist, _AnnotationBase):
             self.patch.set(**bboxprops)
         self._drawFrame = frameon
 
+    @property
+    def xyann(self):
+        return self.xybox
+
+    @xyann.setter
+    def xyann(self, xyann):
+        self.xybox = xyann
+
+    @property
+    def anncoords(self):
+        return self.boxcoords
+
+    @anncoords.setter
+    def anncoords(self, coords):
+        self.boxcoords = coords
+
     def contains(self, event):
         t, tinfo = self.offsetbox.contains(event)
         #if self.arrow_patch is not None:
@@ -1321,7 +1431,9 @@ class AnnotationBbox(martist.Artist, _AnnotationBase):
         return self.prop.get_size_in_points()
 
     def update_positions(self, renderer):
-        "Update the pixel positions of the annotated point and the text."
+        """
+        Update the pixel positions of the annotated point and the text.
+        """
         xy_pixel = self._get_position_xy(renderer)
         self._update_position_xybox(renderer, xy_pixel)
 
@@ -1332,16 +1444,19 @@ class AnnotationBbox(martist.Artist, _AnnotationBase):
             self.arrow_patch.set_mutation_scale(mutation_scale)
 
     def _update_position_xybox(self, renderer, xy_pixel):
-        "Update the pixel positions of the annotation text and the arrow patch."
+        """
+        Update the pixel positions of the annotation text and the arrow
+        patch.
+        """
 
-        x, y = self.xytext
-        if isinstance(self.textcoords, tuple):
-            xcoord, ycoord = self.textcoords
+        x, y = self.xybox
+        if isinstance(self.boxcoords, tuple):
+            xcoord, ycoord = self.boxcoords
             x1, y1 = self._get_xy(renderer, x, y, xcoord)
             x2, y2 = self._get_xy(renderer, x, y, ycoord)
             ox0, oy0 = x1, y2
         else:
-            ox0, oy0 = self._get_xy(renderer, x, y, self.textcoords)
+            ox0, oy0 = self._get_xy(renderer, x, y, self.boxcoords)
 
         w, h, xd, yd = self.offsetbox.get_extent(renderer)
 
@@ -1395,7 +1510,8 @@ class AnnotationBbox(martist.Artist, _AnnotationBase):
 
         if renderer is not None:
             self._renderer = renderer
-        if not self.get_visible(): return
+        if not self.get_visible():
+            return
 
         xy_pixel = self._get_position_xy(renderer)
 
@@ -1451,9 +1567,10 @@ class DraggableBase(object):
     def __init__(self, ref_artist, use_blit=False):
         self.ref_artist = ref_artist
         self.got_artist = False
-        self._use_blit = use_blit
 
         self.canvas = self.ref_artist.figure.canvas
+        self._use_blit = use_blit and self.canvas.supports_blit
+
         c2 = self.canvas.mpl_connect('pick_event', self.on_pick)
         c3 = self.canvas.mpl_connect('button_release_event', self.on_release)
 
@@ -1486,12 +1603,15 @@ class DraggableBase(object):
             if self._use_blit:
                 self.ref_artist.set_animated(True)
                 self.canvas.draw()
-                self.background = self.canvas.copy_from_bbox(self.ref_artist.figure.bbox)
+                self.background = self.canvas.copy_from_bbox(
+                                    self.ref_artist.figure.bbox)
                 self.ref_artist.draw(self.ref_artist.figure._cachedRenderer)
                 self.canvas.blit(self.ref_artist.figure.bbox)
-                self._c1 = self.canvas.mpl_connect('motion_notify_event', self.on_motion_blit)
+                self._c1 = self.canvas.mpl_connect('motion_notify_event',
+                                                   self.on_motion_blit)
             else:
-                self._c1 = self.canvas.mpl_connect('motion_notify_event', self.on_motion)
+                self._c1 = self.canvas.mpl_connect('motion_notify_event',
+                                                   self.on_motion)
             self.save_offset()
 
     def on_release(self, event):
@@ -1504,7 +1624,7 @@ class DraggableBase(object):
                 self.ref_artist.set_animated(False)
 
     def disconnect(self):
-        'disconnect the callbacks'
+        """disconnect the callbacks"""
         for cid in self.cids:
             self.canvas.mpl_disconnect(cid)
 
@@ -1556,30 +1676,30 @@ class DraggableAnnotation(DraggableBase):
 
     def save_offset(self):
         ann = self.annotation
-        x, y = ann.xytext
-        if isinstance(ann.textcoords, tuple):
-            xcoord, ycoord = ann.textcoords
+        x, y = ann.xyann
+        if isinstance(ann.anncoords, tuple):
+            xcoord, ycoord = ann.anncoords
             x1, y1 = ann._get_xy(self.canvas.renderer, x, y, xcoord)
             x2, y2 = ann._get_xy(self.canvas.renderer, x, y, ycoord)
             ox0, oy0 = x1, y2
         else:
-            ox0, oy0 = ann._get_xy(self.canvas.renderer, x, y, ann.textcoords)
+            ox0, oy0 = ann._get_xy(self.canvas.renderer, x, y, ann.anncoords)
 
         self.ox, self.oy = ox0, oy0
-        self.annotation.textcoords = "figure pixels"
+        self.annotation.anncoords = "figure pixels"
         self.update_offset(0, 0)
 
     def update_offset(self, dx, dy):
         ann = self.annotation
-        ann.xytext = self.ox + dx, self.oy + dy
-        x, y = ann.xytext
-        xy = ann._get_xy(self.canvas.renderer, x, y, ann.textcoords)
+        ann.xyann = self.ox + dx, self.oy + dy
+        x, y = ann.xyann
 
     def finalize_offset(self):
-        loc_in_canvas = self.annotation.xytext
-        self.annotation.textcoords = "axes fraction"
-        pos_axes_fraction = self.annotation.axes.transAxes.inverted().transform_point(loc_in_canvas)
-        self.annotation.xytext = tuple(pos_axes_fraction)
+        loc_in_canvas = self.annotation.xyann
+        self.annotation.anncoords = "axes fraction"
+        pos_axes_fraction = self.annotation.axes.transAxes.inverted()
+        pos_axes_fraction = pos_axes_fraction.transform_point(loc_in_canvas)
+        self.annotation.xyann = tuple(pos_axes_fraction)
 
 
 if __name__ == "__main__":
@@ -1602,7 +1722,6 @@ if __name__ == "__main__":
 
     myimage.set_offset((100, 100))
 
-
     myimage2 = OffsetImage(a,
                            zoom=2,
                            norm=None,
@@ -1613,13 +1732,13 @@ if __name__ == "__main__":
                          xybox=(30, 30),
                          xycoords='data',
                          boxcoords="offset points",
-                         frameon=True, pad=0.4, # BboxPatch
+                         frameon=True, pad=0.4,  # BboxPatch
                          bboxprops=dict(boxstyle="round", fc="y"),
                          fontsize=None,
                          arrowprops=dict(arrowstyle="->"),
                          )
 
     ax.add_artist(ann)
-    
+
     plt.draw()
     plt.show()
