@@ -1,4 +1,8 @@
-from __future__ import print_function
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+
+import six
+
 import math
 
 import numpy as np
@@ -34,7 +38,7 @@ class GeoAxes(Axes):
             if rcParams['text.usetex'] and not rcParams['text.latex.unicode']:
                 return r"$%0.0f^\circ$" % degrees
             else:
-                return u"%0.0f\u00b0" % degrees
+                return "%0.0f\u00b0" % degrees
 
     RESOLUTION = 75
 
@@ -158,8 +162,9 @@ class GeoAxes(Axes):
     set_xscale = set_yscale
 
     def set_xlim(self, *args, **kwargs):
-        Axes.set_xlim(self, -np.pi, np.pi)
-        Axes.set_ylim(self, -np.pi / 2.0, np.pi / 2.0)
+        raise TypeError("It is not possible to change axes limits "
+                        "for geographic projections. Please consider "
+                        "using Basemap or Cartopy.")
 
     set_ylim = set_xlim
 
@@ -175,7 +180,7 @@ class GeoAxes(Axes):
             ew = 'E'
         else:
             ew = 'W'
-        return u'%f\u00b0%s, %f\u00b0%s' % (abs(lat), ns, abs(lon), ew)
+        return '%f\u00b0%s, %f\u00b0%s' % (abs(lat), ns, abs(lon), ew)
 
     def set_longitude_grid(self, degrees):
         """
@@ -476,8 +481,16 @@ class MollweideAxes(GeoAxes):
             self._resolution = resolution
 
         def transform_non_affine(self, xy):
-            # MGDTODO: Math is hard ;(
-            return xy
+            x = xy[:, 0:1]
+            y = xy[:, 1:2]
+
+            # from Equations (7, 8) of
+            # http://mathworld.wolfram.com/MollweideProjection.html
+            theta = np.arcsin(y / np.sqrt(2))
+            lon = (np.pi / (2 * np.sqrt(2))) * x / np.cos(theta)
+            lat = np.arcsin((2 * theta + np.sin(2 * theta)) / np.pi)
+
+            return np.concatenate((lon, lat), 1)
         transform_non_affine.__doc__ = Transform.transform_non_affine.__doc__
 
         def inverted(self):
