@@ -41,69 +41,63 @@ xyo = list(zip(xo, yo))
 # Make a list of colors cycling through the rgbcmyk series.
 colors = [colorConverter.to_rgba(c) for c in ('r','g','b','c','y','m','k')]
 
-fig = plt.figure()
+fig, axes = plt.subplots(2,2)
+((ax1, ax2), (ax3, ax4)) = axes # unpack the axes
+
 
-a = fig.add_subplot(2,2,1)
 col = collections.LineCollection([spiral], offsets=xyo,
-                                transOffset=a.transData)
+                                transOffset=ax1.transData)
 trans = fig.dpi_scale_trans + transforms.Affine2D().scale(1.0/72.0)
 col.set_transform(trans)  # the points to pixels transform
     # Note: the first argument to the collection initializer
     # must be a list of sequences of x,y tuples; we have only
     # one sequence, but we still have to put it in a list.
-a.add_collection(col, autolim=True)
+ax1.add_collection(col, autolim=True)
     # autolim=True enables autoscaling.  For collections with
     # offsets like this, it is neither efficient nor accurate,
     # but it is good enough to generate a plot that you can use
     # as a starting point.  If you know beforehand the range of
     # x and y that you want to show, it is better to set them
     # explicitly, leave out the autolim kwarg (or set it to False),
-    # and omit the 'a.autoscale_view()' call below.
+    # and omit the 'ax1.autoscale_view()' call below.
 
 # Make a transform for the line segments such that their size is
 # given in points:
 col.set_color(colors)
 
-a.autoscale_view()  # See comment above, after a.add_collection.
-a.set_title('LineCollection using offsets')
+ax1.autoscale_view()  # See comment above, after ax1.add_collection.
+ax1.set_title('LineCollection using offsets')
 
 
 # The same data as above, but fill the curves.
-
-a = fig.add_subplot(2,2,2)
-
 col = collections.PolyCollection([spiral], offsets=xyo,
-                                transOffset=a.transData)
+                                transOffset=ax2.transData)
 trans = transforms.Affine2D().scale(fig.dpi/72.0)
 col.set_transform(trans)  # the points to pixels transform
-a.add_collection(col, autolim=True)
+ax2.add_collection(col, autolim=True)
 col.set_color(colors)
 
 
-a.autoscale_view()
-a.set_title('PolyCollection using offsets')
+ax2.autoscale_view()
+ax2.set_title('PolyCollection using offsets')
 
 # 7-sided regular polygons
 
-a = fig.add_subplot(2,2,3)
-
 col = collections.RegularPolyCollection(7,
                                         sizes = np.fabs(xx)*10.0, offsets=xyo,
-                                        transOffset=a.transData)
+                                        transOffset=ax3.transData)
 trans = transforms.Affine2D().scale(fig.dpi/72.0)
 col.set_transform(trans)  # the points to pixels transform
-a.add_collection(col, autolim=True)
+ax3.add_collection(col, autolim=True)
 col.set_color(colors)
-a.autoscale_view()
-a.set_title('RegularPolyCollection using offsets')
+ax3.autoscale_view()
+ax3.set_title('RegularPolyCollection using offsets')
 
 
 # Simulate a series of ocean current profiles, successively
 # offset by 0.1 m/s so that they form what is sometimes called
 # a "waterfall" plot or a "stagger" plot.
 
-a = fig.add_subplot(2,2,4)
-
 nverts = 60
 ncurves = 20
 offs = (0.1, 0.0)
@@ -118,14 +112,14 @@ for i in range(ncurves):
     segs.append(curve)
 
 col = collections.LineCollection(segs, offsets=offs)
-a.add_collection(col, autolim=True)
+ax4.add_collection(col, autolim=True)
 col.set_color(colors)
-a.autoscale_view()
-a.set_title('Successive data offsets')
-a.set_xlabel('Zonal velocity component (m/s)')
-a.set_ylabel('Depth (m)')
+ax4.autoscale_view()
+ax4.set_title('Successive data offsets')
+ax4.set_xlabel('Zonal velocity component (m/s)')
+ax4.set_ylabel('Depth (m)')
 # Reverse the y-axis so depth increases downward
-a.set_ylim(a.get_ylim()[::-1])
+ax4.set_ylim(ax4.get_ylim()[::-1])
 
 
 plt.show()
