@@ -4,19 +4,20 @@
 Show how to connect to keypress events
 """
 from __future__ import print_function
+import sys
 import numpy as np
 import matplotlib.pyplot as plt
 
 
 def press(event):
     print('press', event.key)
+    sys.stdout.flush()
     if event.key=='x':
         visible = xl.get_visible()
         xl.set_visible(not visible)
         fig.canvas.draw()
 
-fig = plt.figure()
-ax = fig.add_subplot(111)
+fig, ax = plt.subplots()
 
 fig.canvas.mpl_connect('key_press_event', press)
 
