@@ -65,7 +65,7 @@ The examples below illustrate each of these methods.
 """
 
 from __future__ import print_function
-from matplotlib.pyplot import figure, show
+import matplotlib.pyplot as plt
 from matplotlib.lines import Line2D
 from matplotlib.patches import Rectangle
 from matplotlib.text import Text
@@ -74,15 +74,12 @@ import numpy as np
 from numpy.random import rand
 
 if 1: # simple picking, lines, rectangles and text
-    fig = figure()
-    ax1 = fig.add_subplot(211)
+    fig, (ax1, ax2) = plt.subplots(2,1)
     ax1.set_title('click on points, rectangles or text', picker=True)
     ax1.set_ylabel('ylabel', picker=True, bbox=dict(facecolor='red'))
     line, = ax1.plot(rand(100), 'o', picker=5)  # 5 points tolerance
 
     # pick the rectangle
-    ax2 = fig.add_subplot(212)
-
     bars = ax2.bar(range(10), rand(10), picker=True)
     for label in ax2.get_xticklabels():  # make the xtick labels pickable
         label.set_picker(True)
@@ -140,10 +137,9 @@ if 1: # picking with a custom hit test function
     def onpick2(event):
         print('onpick2 line:', event.pickx, event.picky)
 
-    fig = figure()
-    ax1 = fig.add_subplot(111)
-    ax1.set_title('custom picker for line data')
-    line, = ax1.plot(rand(100), rand(100), 'o', picker=line_picker)
+    fig, ax = plt.subplots()
+    ax.set_title('custom picker for line data')
+    line, = ax.plot(rand(100), rand(100), 'o', picker=line_picker)
     fig.canvas.mpl_connect('pick_event', onpick2)
 
 
@@ -154,20 +150,18 @@ if 1: # picking on a scatter plot (matplotlib.collections.RegularPolyCollection)
         ind = event.ind
         print('onpick3 scatter:', ind, np.take(x, ind), np.take(y, ind))
 
-    fig = figure()
-    ax1 = fig.add_subplot(111)
-    col = ax1.scatter(x, y, 100*s, c, picker=True)
+    fig, ax = plt.subplots()
+    col = ax.scatter(x, y, 100*s, c, picker=True)
     #fig.savefig('pscoll.eps')
     fig.canvas.mpl_connect('pick_event', onpick3)
 
 if 1: # picking images (matplotlib.image.AxesImage)
-    fig = figure()
-    ax1 = fig.add_subplot(111)
-    im1 = ax1.imshow(rand(10,5), extent=(1,2,1,2), picker=True)
-    im2 = ax1.imshow(rand(5,10), extent=(3,4,1,2), picker=True)
-    im3 = ax1.imshow(rand(20,25), extent=(1,2,3,4), picker=True)
-    im4 = ax1.imshow(rand(30,12), extent=(3,4,3,4), picker=True)
-    ax1.axis([0,5,0,5])
+    fig, ax = plt.subplots()
+    im1 = ax.imshow(rand(10,5), extent=(1,2,1,2), picker=True)
+    im2 = ax.imshow(rand(5,10), extent=(3,4,1,2), picker=True)
+    im3 = ax.imshow(rand(20,25), extent=(1,2,3,4), picker=True)
+    im4 = ax.imshow(rand(30,12), extent=(3,4,3,4), picker=True)
+    ax.axis([0,5,0,5])
 
     def onpick4(event):
         artist = event.artist
@@ -179,5 +173,5 @@ if 1: # picking images (matplotlib.image.AxesImage)
     fig.canvas.mpl_connect('pick_event', onpick4)
 
 
-show()
+plt.show()
 
