@@ -36,7 +36,7 @@ from pylab import *
 
 t = arange(0.0, 2.0, 0.01)
 s = sin(2*pi*t)
-ax = subplot(111)
+fig, ax = plt.subplots()
 ax.plot(t, s)
 ax.grid(True)
 
