@@ -14,18 +14,18 @@ This example requires an active internet connection since it uses
 yahoo finance to get the data for plotting
 """
 
-from pylab import figure, show
-from matplotlib.finance import quotes_historical_yahoo
+import matplotlib.pyplot as plt
+from matplotlib.finance import quotes_historical_yahoo_ochl
 from matplotlib.dates import YearLocator, MonthLocator, DateFormatter
 import datetime
-date1 = datetime.date( 1995, 1, 1 )
-date2 = datetime.date( 2004, 4, 12 )
+date1 = datetime.date(1995, 1, 1)
+date2 = datetime.date(2004, 4, 12)
 
-years    = YearLocator()   # every year
-months   = MonthLocator()  # every month
+years = YearLocator()   # every year
+months = MonthLocator()  # every month
 yearsFmt = DateFormatter('%Y')
 
-quotes = quotes_historical_yahoo(
+quotes = quotes_historical_yahoo_ochl(
     'INTC', date1, date2)
 if len(quotes) == 0:
     raise SystemExit
@@ -33,8 +33,7 @@ if len(quotes) == 0:
 dates = [q[0] for q in quotes]
 opens = [q[1] for q in quotes]
 
-fig = figure()
-ax = fig.add_subplot(111)
+fig, ax = plt.subplots()
 ax.plot_date(dates, opens, '-')
 
 # format the ticks
@@ -50,4 +49,4 @@ ax.fmt_ydata = price
 ax.grid(True)
 
 fig.autofmt_xdate()
-show()
+plt.show()
