@@ -1,8 +1,13 @@
-#!/usr/bin/env python2.5
+#!/usr/local/bin/python2.7
+
+from __future__ import print_function
 from pylab import *
+import matplotlib.cbook as cbook
 
 w, h = 512, 512
-s = file('../data/ct.raw', 'rb').read()
+
+datafile = cbook.get_sample_data('ct.raw.gz', asfileobj=True)
+s = datafile.read()
 A = fromstring(s, uint16).astype(float)
 A *= 1.0/max(A)
 A.shape = w, h
@@ -29,4 +34,3 @@ if 0:
     setp(gca(), 'xticklabels', [])
 
 show()
-
