@@ -1,9 +1,9 @@
-#!/usr/bin/env python2.6
+#!/usr/local/bin/python2.7
 """
 You can specify whether images should be plotted with the array origin
 x[0,0] in the upper left or upper right by using the origin parameter.
 You can also control the default be setting image.origin in your
-matplotlibrc file; see http://matplotlib.sourceforge.net/matplotlibrc
+matplotlibrc file; see http://matplotlib.org/matplotlibrc
 """
 from pylab import *
 
