@@ -1,11 +1,15 @@
-#!/usr/bin/env python2.6
+#!/usr/local/bin/python2.7
 # This file generates the matplotlib web page logo
-from pylab import *
 
+from __future__ import print_function
+from pylab import *
+import matplotlib.cbook as cbook
 
 # convert data to mV
-x = 1000*0.1*fromstring(
-    file('../data/membrane.dat', 'rb').read(), float32)
+datafile = cbook.get_sample_data('membrane.dat', asfileobj=False)
+print('loading', datafile)
+
+x = 1000*0.1*fromstring(file(datafile, 'rb').read(), float32)
 # 0.0005 is the sample interval
 t = 0.0005*arange(len(x))
 figure(1, figsize=(7,1), dpi=100)
