@@ -30,7 +30,8 @@ because the locator stores references to the Axis data and view limits
 
 """
 
-from pylab import *
+import matplotlib.pyplot as plt
+import numpy as np
 from matplotlib.ticker import MultipleLocator, FormatStrFormatter
 
 majorLocator   = MultipleLocator(20)
@@ -38,11 +39,11 @@ majorFormatter = FormatStrFormatter('%d')
 minorLocator   = MultipleLocator(5)
 
 
-t = arange(0.0, 100.0, 0.1)
-s = sin(0.1*pi*t)*exp(-t*0.01)
+t = np.arange(0.0, 100.0, 0.1)
+s = np.sin(0.1*np.pi*t)*np.exp(-t*0.01)
 
-ax = subplot(111)
-plot(t,s)
+fig, ax = plt.subplots()
+plt.plot(t,s)
 
 ax.xaxis.set_major_locator(majorLocator)
 ax.xaxis.set_major_formatter(majorFormatter)
@@ -50,4 +51,4 @@ ax.xaxis.set_major_formatter(majorFormatter)
 #for the minor ticks, use no labels; default NullFormatter
 ax.xaxis.set_minor_locator(minorLocator)
 
-show()
+plt.show()
