@@ -22,7 +22,7 @@ minorLocator   = AutoMinorLocator()
 t = np.arange(0.0, 100.0, 0.01)
 s = np.sin(2*np.pi*t)*np.exp(-t*0.01)
 
-ax = plt.subplot(111)
+fig, ax = plt.subplots()
 plt.plot(t,s)
 
 ax.xaxis.set_minor_locator(minorLocator)
