@@ -1,4 +1,4 @@
-#!/usr/bin/env python2.6
+#!/usr/local/bin/python2.7
 """
 pcolormesh uses a QuadMesh, a faster generalization of pcolor, but
 with some restrictions.
@@ -11,7 +11,7 @@ from matplotlib.pyplot import figure, show, savefig
 from matplotlib import cm, colors
 from numpy import ma
 
-n = 56
+n = 12
 x = np.linspace(-1.5,1.5,n)
 y = np.linspace(-1.5,1.5,n*2)
 X,Y = np.meshgrid(x,y);
@@ -28,7 +28,7 @@ Zm = ma.masked_where(np.fabs(Qz) < 0.5*np.amax(Qz), Z)
 fig = figure()
 ax = fig.add_subplot(121)
 ax.set_axis_bgcolor("#bdb76b")
-ax.pcolormesh(Qx,Qz,Z)
+ax.pcolormesh(Qx,Qz,Z, shading='gouraud')
 ax.set_title('Without masked values')
 
 ax = fig.add_subplot(122)
@@ -38,7 +38,7 @@ ax.set_axis_bgcolor("#bdb76b")
 #cmap.set_bad('r', 1.0)
 #ax.pcolormesh(Qx,Qz,Zm, cmap=cmap)
 #  Or use the default, which is transparent:
-col = ax.pcolormesh(Qx,Qz,Zm)
+col = ax.pcolormesh(Qx,Qz,Zm,shading='gouraud')
 ax.set_title('With masked values')
 
 
