@@ -1,5 +1,6 @@
-#!/usr/bin/env python2.6
+#!/usr/local/bin/python2.7
 # -*- noplot -*-
+
 """
 N       Classic     Base renderer    Ext renderer
 20       0.22           0.14            0.14
@@ -8,15 +9,16 @@ N       Classic     Base renderer    Ext renderer
 10000    3.30           1.31            0.53
 50000    19.30          6.53            1.98
 """
-from pylab import *
+from __future__ import print_function
+import pylab
 
 import time
 
 
 for N in (20,100,1000,10000,50000):
     tstart = time.time()
-    x = 0.9*rand(N)
-    y = 0.9*rand(N)
-    s = 20*rand(N)
-    scatter(x,y,s)
-    print '%d symbols in %1.2f s' % (N, time.time()-tstart)
+    x = 0.9*pylab.rand(N)
+    y = 0.9*pylab.rand(N)
+    s = 20*pylab.rand(N)
+    pylab.scatter(x,y,s)
+    print ('%d symbols in %1.2f s' % (N, time.time()-tstart))
