@@ -1,4 +1,4 @@
-#!/usr/bin/env python2.7
+#!/usr/local/bin/python2.7
 from pylab import *
 
 dt = 0.0005
@@ -25,5 +25,6 @@ Fs = int(1.0/dt)  # the sampling frequency
 ax1 = subplot(211)
 plot(t, x)
 subplot(212, sharex=ax1)           
-Pxx, freqs, bins, im = specgram(x, NFFT=NFFT, Fs=Fs, noverlap=900)
+Pxx, freqs, bins, im = specgram(x, NFFT=NFFT, Fs=Fs, noverlap=900, 
+                                cmap=cm.gist_heat)
 show()
