@@ -1,4 +1,6 @@
-#!/usr/bin/env python2.5
+#!/usr/local/bin/python2.7
+
+from __future__ import unicode_literals
 
 import os, sys, re
 
@@ -16,9 +18,7 @@ stests = [
     r'$\mathscr{Script}$']
 
 if sys.maxunicode > 0xffff:
-    stests.append(
-        ur'Direct Unicode: $\u23ce \mathrm{\ue0f2 \U0001D538}$'
-    )
+    s = r'Direct Unicode: $\u23ce \mathrm{\ue0f2 \U0001D538}$'
 
 from pylab import *
 
