@@ -6,7 +6,7 @@ from data_helper import get_two_stock_data
 
 d1, p1, d2, p2 = get_two_stock_data()
 
-ax = subplot(111)
+fig, ax = subplots()
 lines = plot(d1, p1, 'bs', d2, p2, 'go')
 xlabel('Days')
 ylabel('Normalized price')
