@@ -1,8 +1,7 @@
 #!/usr/local/bin/python2.7
 """
 Use backend agg to access the figure canvas as an RGB string and then
-convert it to an array and pass the string it to PIL for
-rendering
+convert it to an array and pass it to Pillow for rendering.
 """
 
 import pylab
@@ -30,5 +29,7 @@ X = pylab.fromstring(s, pylab.uint8)
 X.shape = h, w, 3
 
 im = Image.fromstring( "RGB", (w,h), s)
-im.show()
 
+# Uncomment this line to display the image using ImageMagick's
+# `display` tool.
+# im.show()
