@@ -10,8 +10,15 @@
 # may be distributed without limitation.
 
 from __future__ import unicode_literals
-import sys, os, random
-from PyQt4 import QtGui, QtCore
+import sys
+import os
+import random
+from matplotlib.backends import qt4_compat
+use_pyside = qt4_compat.QT_API == qt4_compat.QT_API_PYSIDE
+if use_pyside:
+    from PySide import QtGui, QtCore
+else:
+    from PyQt4 import QtGui, QtCore
 
 from numpy import arange, sin, pi
 from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
@@ -57,15 +64,15 @@ class MyDynamicMplCanvas(MyMplCanvas):
     def __init__(self, *args, **kwargs):
         MyMplCanvas.__init__(self, *args, **kwargs)
         timer = QtCore.QTimer(self)
-        QtCore.QObject.connect(timer, QtCore.SIGNAL("timeout()"), self.update_figure)
+        timer.timeout.connect(self.update_figure)
         timer.start(1000)
 
     def compute_initial_figure(self):
-         self.axes.plot([0, 1, 2, 3], [1, 2, 0, 4], 'r')
+        self.axes.plot([0, 1, 2, 3], [1, 2, 0, 4], 'r')
 
     def update_figure(self):
         # Build a list of 4 random integers between 0 and 10 (both inclusive)
-        l = [ random.randint(0, 10) for i in range(4) ]
+        l = [random.randint(0, 10) for i in range(4)]
 
         self.axes.plot([0, 1, 2, 3], l, 'r')
         self.draw()
