@@ -1,4 +1,4 @@
-#!/usr/bin/env python2.7
+#!/usr/local/bin/python2.7
 """
 This is an example that shows you how to work directly with the agg
 figure canvas to create a figure using the pythonic API.
@@ -54,7 +54,7 @@ if 0:
 
 if 0:
     # pass off to PIL
-    import Image
+    from PIL import Image
     im = Image.fromstring( "RGB", (w,h), s)
     im.show()
 
